/*
 * AlienInterface
 *
 * Back end infrastructure for communicating with aliens.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include <sstream>
#include <string>

#include "Definitions.h"
#include "KeyPressed.h"
#include "NumberGenerator.h"
#include "Ship.h"
#include "Coords.h"
#include "SoundSystem.h"
#include "Artifact.h"
#include "Star.h"
#include "View.h"
#include "Information.h"
#include "AlienInterface.h"


using namespace std;

int AlienInterface::_page;
int AlienInterface::_vpos;
int AlienInterface::_selected_option;
bool AlienInterface::_new_page;
Artifact AlienInterface::_interesting_artifact;
int AlienInterface::_alien_type;

/*
 * Constructor.
 */
AlienInterface::AlienInterface() {
	_page=0;
	_selected_option=0;
	_new_page=true;
	_interesting_artifact=Artifact::nullArtifact();
	_alien_type=-1;
}

/*
 * Takes action if a key is pressed in the interface.
 */
void AlienInterface::keyPressed(KeyPressed kp) {
	stringstream ss;
	ss.precision(0);

	Ship *my_ship = Ship::getMyShip();
	_alien_type = (*my_ship).getShipStar().getAlienType();

	int keypad = kp.getKeyPressed();
	int mouse_pressed = kp.isMousePressed();



	if (keypad==KEY_HOME) {
		_page=0;
		_new_page=true;
		(*my_ship).setVisitingAliens(false);
		SoundSystem::stopSound((*my_ship).getShipStar().getAlienType());
		SoundSystem::resumeMusic();
		_selected_option=0;
	} else if ((keypad==KEY_ENTER)||(mouse_pressed==1)) {
		_page++;
		_new_page=true;

		if (_page==1) {
			_interesting_artifact = Artifact::getInterestingArtifact(_alien_type);
			_selected_option=0;
		}

		if (_page==2) {
			if (_interesting_artifact==Artifact::nullArtifact()) {
				//No Artifact - Skip to sales screen
				_page++;
			} else if (_selected_option==0) {
				//Refused Artifact - Skip to Sales Screen
				_page++;
			}
			_selected_option=0;
		}

		if (_page==4) {
			_page--;

			vector<Artifact> found_artifacts = Artifact::getFoundArtifacts();

			int upgrade1_cursor_position=Artifact::getFoundArtifacts().size();
			int upgrade2_cursor_position=upgrade1_cursor_position+1;
			int fuel_cursor_position=upgrade1_cursor_position+2;

			if ((*my_ship).getUpgrades(2*_alien_type)>=10) {
				//Upgrade1 maxed out, shuffle up one.
				upgrade1_cursor_position=-999;
				upgrade2_cursor_position--;
				fuel_cursor_position--;
			}

			if ((*my_ship).getUpgrades(2*_alien_type+1)>=10) {
				//Upgrade2 maxed out, shuffle up one.
				upgrade2_cursor_position=-999;
				fuel_cursor_position--;
			}


			if (_selected_option<(int)found_artifacts.size()) {
				//Sell Artifact
				(*my_ship).addCash(found_artifacts[_selected_option].getArtifactCashValue(_alien_type));
				found_artifacts[_selected_option].removeFromFoundArtifacts();
			} else if (_selected_option==upgrade1_cursor_position) {
				//Upgrade 1
				if ((*my_ship).getCash()>=(1+(*my_ship).getUpgrades(2*_alien_type))*10) {
					(*my_ship).addCash(-(1+(*my_ship).getUpgrades(2*_alien_type))*10);
					(*my_ship).incrementUpgrades(2*_alien_type);
				}
			} else if (_selected_option==upgrade2_cursor_position) {
				//Upgrade 2
				if ((*my_ship).getCash()>=(1+(*my_ship).getUpgrades(2*_alien_type+1))*10) {
					(*my_ship).addCash(-(1+(*my_ship).getUpgrades(2*_alien_type+1))*10);
					(*my_ship).incrementUpgrades(2*_alien_type+1);
				}
			} else if (_selected_option==fuel_cursor_position) {
				//Fuel
				_new_page=false;
				float mult=10.0;
				while (true) {
					if ((*my_ship).getCash()<0.01) {
						break;
					}

					if (((*my_ship).getMaxFuel()-(*my_ship).getFuel())>mult) {
						float remain = ((*my_ship).getMaxFuel()-(*my_ship).getFuel());
						if (((*my_ship).getCash()>=mult) ||((*my_ship).getCash()>=remain))  {
							if (remain>mult) {
								(*my_ship).addCash(-mult);
								(*my_ship).addFuel(mult);
							} else {
								(*my_ship).addCash(-remain);
								(*my_ship).addFuel(remain);
							}
							break;
						}
					}
					mult/=10;
					if (mult<=0.001) {
						(*my_ship).addCash(-((*my_ship).getMaxFuel()-(*my_ship).getFuel()));
						(*my_ship).setFuel((*my_ship).getMaxFuel());
						break;
					}
				}
			} else {
				//Exit
				_page=0;
				_selected_option=0;
				_alien_type=-1;
				(*my_ship).setVisitingAliens(false);
				SoundSystem::stopSound(_alien_type);
				SoundSystem::resumeMusic();
			}
		}
	} else if ((keypad==KEY_8)||(mouse_pressed==4)) {
		//Up
		vector<string> options = AlienInterface::getOptions();
		_selected_option--;
		if (_selected_option<0)
			_selected_option=options.size()-1;
	} else if ((keypad==KEY_2)||(mouse_pressed==5)) {
		//Down
		vector<string> options = AlienInterface::getOptions();
		_selected_option++;
		if (_selected_option>=(int)options.size())
			_selected_option=0;
	}
}

/*
 * Gets messages to display on AlienInterfaceSurface
 */
vector<string> AlienInterface::getMessage() {
	vector<string> message;

	Ship *my_ship = Ship::getMyShip();

	string star_name = (*my_ship).getShipStar().getStarName();

	string *alien_types = Artifact::getAlienTypes();

	if (_page==0) {
		//Intro Screen
		message.push_back("You are being hailed from an");
		message.push_back("alien world orbiting "+star_name+"....");
	} else if (_page==1) {
		message.push_back("We are the "+alien_types[_alien_type]+"s.");
		message.push_back("");

		if (_interesting_artifact==Artifact::nullArtifact()) {
			message.push_back("You are welcome to visit our planet.");
		} else {
			message.push_back("You have an artifact of");
			switch(_interesting_artifact.getArtifactValue()) {
				case 0:
				message.push_back("some interest to us.");
				break;
				case 1:
				message.push_back("interest to us.");
				break;
				case 2:
				message.push_back("great interest to us!");
				break;
			}
			message.push_back("");
			message.push_back("Would you be willing to trade");
			message.push_back("it for information?");
		}
	} else if (_page==2) {
		View::setUpdateOverride();
		_interesting_artifact.removeFromFoundArtifacts();

		switch(_interesting_artifact.getArtifactValue()) {
			case 0:
				message.push_back("I'm sure we have space for this");
				message.push_back("somewhere in the archives...");
				break;
			case 1:
				message.push_back("Thank you for sharing this");
				message.push_back("fine treasure with us.");
				break;
			case 2:
				message.push_back("This will become one of the finest");
				message.push_back("artifacts in our collection!");
				break;
		}

		message.push_back("");
		message.push_back("Now, for your information...");
		message.push_back("");

		Coords delta = Artifact::getPrizeStar().getPosition()-(*my_ship).getShipCoords();

		float radius = delta.getLength();
		float innerRadius[3];
		float outerRadius[3];
		int ancient_fraction[3] = {3,5,10};
		for (int i=0;i<3;i++) {
			float radiusDelta = NumberGenerator::galaxy_rand((*my_ship).getShipStar().getStarNumber(),20+i)*(2.55*radius/ancient_fraction[i]);
			innerRadius[i]=(radius-radiusDelta);
			outerRadius[i]=1+(innerRadius[i]+(2.55*radius/ancient_fraction[i]));
		}

		float angle = delta.getAngle();

		float wrong_angle = angle+ (1+((int)(NumberGenerator::galaxy_rand((*my_ship).getShipStar().getStarNumber(),20)*3)))*PI/2.0;

		int *ship_location = (*my_ship).getShipLocation();
		int *prize_location = Artifact::getPrizeLocation();

		int exclusion_location_prefix = 0;
		for (int i=0;i<7;i++) {
			if (prize_location[i]!=ship_location[i])
				break;
			exclusion_location_prefix=exclusion_location_prefix*10+ship_location[i];
		}
		float exclusion_location = (float)exclusion_location_prefix;
		while((exclusion_location>0)&&(exclusion_location<1000000))
			exclusion_location*=10;

		int poor_location_prefix = 0;
		int invalid_count=2;
		for (int i=0;i<7;i++) {
			poor_location_prefix=poor_location_prefix*10+ship_location[i];
			if ((i==6)&&(ship_location[i]==prize_location[i])) {
				//Oh shit!  They're cashing in a cheap artifact in the sector the prize is in.
				if (ship_location[i]<5) {
					poor_location_prefix++;
				} else {
					poor_location_prefix--;
				}
			}

			if (invalid_count<1)
				invalid_count--;
			else if (prize_location[i]!=ship_location[i])
				invalid_count--;
			if (invalid_count==0)
				break;
		}
		float poor_location = (float)poor_location_prefix;
		while(poor_location<1000000)
			poor_location*=10;

		int medium_location_prefix = 0;
		for (int i=0;i<7;i++) {
			medium_location_prefix=medium_location_prefix*10+ship_location[i];
			if ((i==6)&&(ship_location[i]==prize_location[i])) {
				//Oh shit!  They're cashing in a cheap artifact in the sector the prize is in.
				if (ship_location[i]<5) {
					medium_location_prefix++;
				} else {
					medium_location_prefix--;
				}
			}
			if (prize_location[i]!=ship_location[i])
				break;
		}
		float medium_location = (float)medium_location_prefix;
		while(medium_location<1000000)
			medium_location*=10;

		int high_location_prefix = 0;
		for (int i=0;i<7;i++) {
			high_location_prefix=high_location_prefix*10+prize_location[i];
			if (prize_location[i]!=ship_location[i])
				break;
		}
		float high_location = (float)high_location_prefix;
		while(high_location<1000000)
			high_location*=10;

		stringstream ss;
		ss.precision(0);
		switch (_alien_type*3+_interesting_artifact.getArtifactValue()) {
			case 0:
			case 1:
			case 2:
				//Ancients - Low/Med/High value
				int val = _interesting_artifact.getArtifactValue();
				ss << "The Prize is between " << fixed << innerRadius[val] << " and";
				message.push_back(ss.str());
				ss.str(string());
				ss << fixed << outerRadius[val] << " light years from here.";
				message.push_back(ss.str());
				Information::addInformation(Information(0,innerRadius[val]));
				Information::addInformation(Information(1,outerRadius[val]));
				break;

			case 3:
				//Industrialists - Low Value
				message.push_back("The Prize is not "+Coords::getCoarseAngleName(wrong_angle)+" of here.");
				Information::addInformation(Information(4,Coords::roundAngleDownCoarse(wrong_angle),Coords::roundAngleUpCoarse(wrong_angle)));
				break;
			case 4:
				//Industrialists - Medium Value
				message.push_back("The Prize is "+Coords::getCoarseAngleName(angle)+" of here.");
				Information::addInformation(Information(3,Coords::roundAngleDownCoarse(angle),Coords::roundAngleUpCoarse(angle)));
				break;
			case 5:
				//Industrialists - High Value
				message.push_back("The Prize is due "+Coords::getFineAngleName(angle)+" of here.");
				Information::addInformation(Information(3,Coords::roundAngleDownFine(angle),Coords::roundAngleUpFine(angle)));
				break;
			case 6:
				//Futurists - Low Value
				if (exclusion_location_prefix!=0) {
					ss << exclusion_location_prefix;
					message.push_back("You're getting closer.");
					message.push_back("");
					message.push_back("The prize is in a sector");
					message.push_back("with prefix "+ss.str()+".");
					message.push_back("");
				}
				ss.str(string());
				ss << poor_location_prefix;
				message.push_back("The prize is not in a sector");
				message.push_back("with prefix "+ss.str()+".");
				Information::addInformation(Information(6,poor_location));
				if (exclusion_location_prefix!=0) {
					Information::addInformation(Information(7,exclusion_location));
				}
				break;
			case 7:
				//Futurists - Medium Value
				if (exclusion_location_prefix!=0) {
					ss << exclusion_location_prefix;
					message.push_back("You're getting closer.");
					message.push_back("");
					message.push_back("The prize is in a sector");
					message.push_back("with prefix "+ss.str()+".");
					message.push_back("");
				}
				ss.str(string());
				ss << medium_location_prefix;
				message.push_back("The prize is not in a sector");
				message.push_back("with prefix "+ss.str()+".");
				Information::addInformation(Information(6,medium_location));
				if (exclusion_location_prefix!=0) {
					Information::addInformation(Information(7,exclusion_location));
				}
				break;
			case 8:
				//Futurists - High Value
				ss << high_location_prefix;
				message.push_back("The prize is in a sector");
				message.push_back("with prefix "+ss.str()+".");
				Information::addInformation(Information(7,high_location));
				break;
		}
	} else if (_page==3) {
		message.push_back("Would you like to sell any");
		message.push_back("of your other artifacts, or");
		message.push_back("refuel or upgrade your ship?");
	}

	return message;

}

/*
 * Get options to display on AlienInterfaceSurface
 */
vector<string> AlienInterface::getOptions() {
	vector<string> options;

	Ship *my_ship = Ship::getMyShip();

	string *upgrade_name = Ship::getUpgradeName();

	switch (_page) {
		case 1:
			if (_interesting_artifact!=Artifact::nullArtifact()) {
				options.push_back("No");
				options.push_back("Yes");
			}
			break;
		case 3:
			_alien_type = (*my_ship).getShipStar().getAlienType();
			vector<Artifact> found_artifacts = Artifact::getFoundArtifacts();
			stringstream cash_value_ss;
			stringstream upgrade_value_ss;

			for (int i=0;i<(int)found_artifacts.size();i++) {
				cash_value_ss.str(string());
				cash_value_ss << found_artifacts[i].getArtifactCashValue(_alien_type);
				stringstream colorss;
				colorss << "@"<<found_artifacts[i].getArtifactType();
				options.push_back(colorss.str()+"Sell '"+found_artifacts[i].getArtifactName()+"' for M$"+cash_value_ss.str());
			}

			int upgrade = (*my_ship).getUpgrades(2*_alien_type);
			if (upgrade<10) {
				cash_value_ss.str(string());
				cash_value_ss << (upgrade+1)*10;
				upgrade_value_ss.str(string());
				upgrade_value_ss << upgrade+1;
				options.push_back("Buy "+upgrade_name[2*_alien_type]+" "+upgrade_value_ss.str()+" for M$"+cash_value_ss.str());
			}

			upgrade = (*my_ship).getUpgrades(2*_alien_type+1);
			if (upgrade<10) {
				cash_value_ss.str(string());
				cash_value_ss << (upgrade+1)*10;
				upgrade_value_ss.str(string());
				upgrade_value_ss << upgrade+1;
				options.push_back("Buy "+upgrade_name[2*_alien_type+1]+" "+upgrade_value_ss.str()+" for M$"+cash_value_ss.str());
			}

			options.push_back("Buy Fuel (M$1.00/L)");
			options.push_back("Done");

			break;
	}

	return options;

}


/*
 * Get the vertical position to place the message on the AlienInterfaceSurface
 */
int AlienInterface::getMessageVPos() {
	switch (_page) {
		case 3:
			return 10;
		default:
			break;
	}

	return -1;
}

/*
 * Get the vertical position to place the options on the AlienInterfaceSurface
 */
int AlienInterface::getOptionsVPos() {
	switch (_page) {
		case 1:
			return 420;
		case 3:
			return 110;
		default:
			break;
	}

	return -1;
}

/*
 * Returns the option presently marked by the cursor.
 */
int AlienInterface::getSelectedOption() {
	return _selected_option;
}

/*
 * Checks to see if the page is new, resets _new_page to false afterward.
 */
bool AlienInterface::isNewPage() {
	if (_new_page) {
		_new_page = false;
		return true;
	}
	return false;
}

/*
 * Reset this interface, in case the game is won.
 */
void AlienInterface::reset() {
   	_page=0;
	_selected_option=0;
	_new_page=true;
	_interesting_artifact=Artifact::nullArtifact();
	_alien_type=-1;
}
