/*
 * Artifact
 *
 * Class representing alien artifacts, with helper functions.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef ARTIFACT_H
#define ARTIFACT_H

#include <string>
#include <sstream>
#include <vector>

class Star;

class Artifact {
private:
	int _star_number;

	static std::vector<Artifact> _found_artifacts;
	static bool _updated;
	static Star _prize_star;
	static int _prize_location[7];
	static Artifact _prize_artifact;
	static std::string _artifact_message;
	static bool _between_games;
	static std::string _alien_types[3];
public:
	Artifact();
	Artifact(const Artifact &a);
	Artifact(int star_number);
	bool operator==(const Artifact &a);
	bool operator!=(const Artifact &a);
	static Artifact nullArtifact();
	std::string getArtifactName();
	int getArtifactNumber();
	int getArtifactCashValue(int alien_type);
	int getArtifactType();
	int getArtifactValue();
	static std::vector<Artifact> getFoundArtifacts();
	static void addFoundArtifact(Artifact a);
	static bool isUpdated();
	static void initPrizeLocation();
	static Artifact getPrizeArtifact();
	static Star getPrizeStar();
	static int *getPrizeLocation();
	bool isPrizeArtifact();
	static Artifact getInterestingArtifact(const int &alien_type);
	void removeFromFoundArtifacts();
	static void reset();
	static void createArtifactMessage();
	static std::string getArtifactMessage();
	static void handleArtiMessage(std::string message);
	static bool isBetweenGames();
	static void setBetweenGames(bool between_games);
	static std::string *getAlienTypes();
};

#endif
