/*
 * CaptainNames
 *
 * Class to distribute ship captain names to clients, and tools to ensure names do
 * not overlap.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 *
 */

#include <string>
#include <iostream>

#include "NumberGenerator.h"
#include "CaptainNames.h"

using namespace std;

vector<string> CaptainNames::_available_names;
list<string> CaptainNames::_used_names;

/*
 * Array of Captain Names.
 * Names are alphabetized, but contain names of ancient explorers, astronauts, and fictional
 * sci-fi adventurers.  Note that NUM_CAPTAIN_NAMES is defined in CaptainNames.h.
 */
string CaptainNames::_captain_names_array[NUM_CAPTAIN_NAMES] = {
	"Adama",
	"Aldrin",
	"Anders",
	"Armstrong",
	"Arroway",
	"Atredies",
	"Balboa",
	"Bean",
	"Beeblebrox",
	"Belzoni",
	"Bender",
	"Bering",
	"Blake",
	"Boming",
	"Bondar",
	"Borman",
	"Bowman",
	"Branson",
	"Byrd",
	"Cabot",
	"Cartier",
	"Cavendish",
	"Cernan",
	"Chaffee",
	"Clark",
	"Collins",
	"Columbus",
	"Conrad",
	"Cook",
	"Cooper",
	"Cortez",
	"Crippen",
	"Dent",
	"Drake",
	"Duke",
	"Ericson",
	"Erik the Red",
	"Floyd",
	"Ford",
	"Fry",
	"Fwiffo",
	"Gagarin",
	"Garibaldi",
	"Garneau",
	"Garriot",
	"Glenn",
	"Golombek",
	"Grissom",
	"Hadfield",
	"Haipeng",
	"Haise",
	"Haisheng",
	"Hanno",
	"Harshaw",
	"Hermaszewski",
	"Heyerdahl",
	"Hillary",
	"Hodgson",
	"Hudson",
	"Irwin",
	"Ivonova",
	"Jackson",
	"Jameson",
	"Janeway",
	"Junlong",
	"Kirk",
	"Koenig",
	"Leakey",
	"Leela",
	"Leonov",
	"Lewis",
	"Livingstone",
	"Liwei",
	"Lovell",
	"Lucid",
	"MacKenzie",
	"Magellan",
	"Malaspina",
	"Mattingly",
	"McKay",
	"Mitchell",
	"Moresby",
	"Musgrave",
	"Nelson",
	"Norton",
	"Nye",
	"O'Neill",
	"Perez",
	"Piccard",
	"Pizarro",
	"Polo",
	"Poole",
	"Pytheas",
	"Reiche",
	"Resnik",
	"Reynolds",
	"Rogers",
	"Sagan",
	"Salak",
	"Saotome",
	"Schirra",
	"Schmitt",
	"Schweickart",
	"Scott",
	"Seldon",
	"Shaeffer",
	"Shepard",
	"Sheridan",
	"Sinclair",
	"Sisko",
	"Slayton",
	"Spiegel",
	"Squires",
	"Stafford",
	"Stanley",
	"Sullivan",
	"Swigert",
	"Tasman",
	"Tereshkova",
	"Thagard",
	"Tigh",
	"Titov",
	"Tracy",
	"Tryggvason",
	"Tyson",
	"Vancouver",
	"Vespucci",
	"Voss",
	"Wiggin",
	"White",
	"Young",
	"Zelnick",
	"Zhigang",
	"da Gama",
	"de Champlain",
	"de Coronado",
	"de Soto"
};

/*
 * Constuctor.  Initializes the _available_names array, which will be used to ensure two
 * players are not given the same name in a multiplayer game.
 */
CaptainNames::CaptainNames() {
	for (int i=0;i<NUM_CAPTAIN_NAMES;i++) {
		_available_names.push_back(_captain_names_array[i]);
	}
}

/*
 * Get a random name, and remove it from the list of available names.
 */
string CaptainNames::getRandomName() {
	int selection = ((int)(NumberGenerator::rand0()*_available_names.size()))%_available_names.size();
	string name = _available_names[selection];
	_used_names.push_back(name);
	_available_names.erase(_available_names.begin()+selection,_available_names.begin()+selection+1);
	return name;
}

/*
 * Returns a Captain Name to the list of available names.  Used when the player quits the game.
 */
void CaptainNames::returnName(string name) {
	_available_names.push_back(name);
	_used_names.remove(name);
}
