/*
 * CargoSurface
 *
 * Renders collected artifacts to the user interface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <sstream>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "CargoSurface.h"
#include "Definitions.h"
#include "Font.h"
#include "Ship.h"
#include "Artifact.h"
#include "ScreenSurface.h"

using namespace std;

SDL_Surface *CargoSurface::_cargo_icons;
SDL_Surface *CargoSurface::_cargo_hold;

/*
 * Constructor.  Must be called before blit().
 */
CargoSurface::CargoSurface() {
	_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_WIDTH-SCREEN_CENTER*2, 34, SCREEN_DEPTH,0,0,0,0);

	_rect.x=SCREEN_CENTER*2;
	_rect.y=SCREEN_HEIGHT-84-34;
	_rect.w=SCREEN_WIDTH-SCREEN_CENTER*2;
	_rect.h=34;

	_cargo_icons = SDL_LoadBMP("Sheets/CargoIcons.bmp");
	_cargo_hold = SDL_LoadBMP("Sheets/CargoHold.bmp");
}

/*
 * Destructor.  Frees surfaces.
 */
CargoSurface::~CargoSurface() {
	SDL_FreeSurface(_surface);
	SDL_FreeSurface(_cargo_icons);
	SDL_FreeSurface(_cargo_hold);
}

/*
 * Draws the cargo space to the screen surface.  Displays the background cargo hold image,
 * cargo icons, or blank spaces if the hold is empty.
 */
void CargoSurface::blit() {
	if (Artifact::isUpdated()) {
		vector<Artifact> found_artifacts = Artifact::getFoundArtifacts();

		string message;
		string artifact_name;
		int i;

		SDL_Rect cargo_icons_rect, cargo_location_rect;

		cargo_icons_rect.x=0;
		cargo_icons_rect.y=0;
		cargo_icons_rect.w=30;
		cargo_icons_rect.h=30;

		cargo_location_rect.x=0;
		cargo_location_rect.y=2;
		cargo_location_rect.w=30;
		cargo_location_rect.h=30;


		SDL_BlitSurface(_cargo_hold,NULL,_surface,NULL);

		Ship *my_ship = Ship::getMyShip();

		for(i=0;i<10;i++) {
			if (i<(int)found_artifacts.size()) {

				cargo_icons_rect.x=90*found_artifacts[i].getArtifactType()+30*found_artifacts[i].getArtifactValue();
				cargo_location_rect.x=1+32*i;
				SDL_BlitSurface(_cargo_icons,&cargo_icons_rect,_surface,&cargo_location_rect);
			} else if (i<(*my_ship).getCargoHoldSize()) {
				boxColor(_surface,9+32*i,9,-9+32*(i+1),23,0x4C4C4CFF);
			}
		}

		rectangleColor(_surface,0,0,SCREEN_CENTER*4/3-1,49,0x3F3FFFFF);
	}

	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
}
