/*
 * Coords
 *
 * Class to represent an (x,y) vector, with helper functions.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <math.h>
#include <string>
#include "SDL/SDL.h"

#ifndef COORDS_H
#define COORDS_H

class Coords {
private:
	float _x,_y;
	static std::string _coarse_angle_names[4];
	static std::string _fine_angle_names[8];
	static float _coarse_angle_values[4];
	static float _fine_angle_values[8];
public:
	Coords();
	Coords(const Coords &c);
	Coords(const float x, const float y);
	Coords(const float r, const float theta, const bool polar);
	bool operator== (const Coords &b);
	bool operator!= (const Coords &b);
	Coords operator+ (const Coords &b);
	Coords operator- (const Coords &b);
	Coords operator* (const float &d);
	Coords operator/ (const float &d);
	void operator+= (const Coords &b);
	void operator-= (const Coords &b);
	float getX();
	float getY();
	float getAngle();
	float getLength();
	void getIntValues(int *x_pos, int *y_pos, float scale);
	void getIntValues(int *x_pos, int *y_pos, float scale, int shrink);
	void getIntValues(Sint16 *x_pos, Sint16 *y_pos, float scale);
	void getIntValues(Sint16 *x_pos, Sint16 *y_pos, float scale, int shrink);
	static std::string getCoarseAngleName(float angle);
	static std::string getFineAngleName(float angle);
	static float roundAngleUpCoarse(float angle);
	static float roundAngleDownCoarse(float angle);
	static float roundAngleUpFine(float angle);
	static float roundAngleDownFine(float angle);
	std::string toString();
	static float getBackAngle(float angle1, float angle2);
	static bool angleBetween(float angle1, float angle2, float angle3);
};

#endif
