/*
 * Font
 *
 * Helper class to handle both SDL_gfx and SDL_ttf fonts.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <fstream>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Font.h"

using namespace std;

char *Font::_font_data[3];
TTF_Font *Font::_ttf_font_data[3];

/*
 * Constructor. Must be initialized before any fonts are rendered.
 */
Font::Font() {
	TTF_Init();
}

/*
 * Set SDL_gfx font.  Values for font can be "default" or the name of an fnt file contained
 * in the Fonts directory.
 */
void Font::setFont(string font) {
	int w,h,font_num,size;
	w=h=8;
	font_num=0;
	FILE *file;
	string filename = "Fonts/"+font;
	font_num=-1;
	size=10240;

	if (font=="default") {
		gfxPrimitivesSetFont(NULL,8,8);
	} else {
		if (font=="10x20.fnt") {
			size=10240;
			w=10;
			h=20;
			font_num=0;
		} else if (font=="9x15B.fnt") {
			size=7680;
			w=9;
			h=15;
			font_num=1;
		} else if (font=="9x18B.fnt") {
			size=9216;
			w=9;
			h=18;
			font_num=2;
		}

		if (font_num>-1) {
			if (_font_data[font_num]==NULL) {
				file = fopen(filename.c_str(),"r");
				_font_data[font_num]=(char *)malloc(size);
				fread(_font_data[font_num],size,1,file);
				fclose(file);
			}

			gfxPrimitivesSetFont(_font_data[font_num],w,h);
		}
	}

}

/*
 * Returns at SDL_ttf font basd on the font file name.
 */
TTF_Font *Font::getTTFFont(string font) {
	int font_num=-1;
	int font_size=-1;
	if (font=="Burtinomatic.ttf") {
		font_num=0;
		font_size=30;
	} else if (font=="Incognitype.ttf") {
		font_num=1;
		font_size=20;
	} else if (font=="HookedUp.ttf") {
		font_num=2;
		font_size=30;
	}

	if (font_num>-1) {
		if (_ttf_font_data[font_num]==NULL) {
			_ttf_font_data[font_num]=TTF_OpenFont( ("Fonts/"+font).c_str(), font_size );
		}

		return _ttf_font_data[font_num];
	}

	return NULL;
}
