/*
 * GalacticArtifact
 *
 * Main program for both client and server.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#if defined(__MINGW32__)
#define sleep(t) _sleep((t) * 1000)
#endif

#ifndef __MINGW32__
#include <sys/signal.h>
#endif

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <fstream>
#include <vector>

#include "Definitions.h"
#include "NumberGenerator.h"
#include "Font.h"
#include "SoundSystem.h"
#include "FrameTimer.h"
#include "Coords.h"
#include "Artifact.h"
#include "View.h"
#include "Ship.h"
#include "Star.h"
#include "Messages.h"
#include "Information.h"
#include "ScreenSurface.h"
#include "ShipSurface.h"
#include "CargoSurface.h"
#include "MessageSurface.h"
#include "MinimapSurface.h"
#include "OverviewSurface.h"
#include "GalaxySurface.h"
#include "RadarSurface.h"
#include "KeyPressed.h"
#include "MessageBuffer.h"
#include "AlienInterface.h"
#include "AlienInterfaceSurface.h"
#include "VictorySurface.h"
#include "VerifyQuitSurface.h"
#include "MenuInterface.h"
#include "MenuInterfaceSurface.h"
#include "InformationSurface.h"
#include "GameServer.h"
#include "NetworkConnection.h"
#include "NetworkEvent.h"

using namespace std;

/*
 * Start the game server.  Run with --server.
 * You may want to look in Definitions.h to set game server port and name.
 */
void startServer() {
	SDL_Init(SDL_INIT_TIMER);
#ifndef __MINGW32__
	//ignore SIGPIPE errors (otherwise server crashes!!!)
	signal(SIGPIPE, SIG_IGN);
#endif

	GameServer gameServer;
	if (!gameServer.initHost()) {
		cout << "Server is already running!"<<endl;
		exit(0);
	}
	int minutes15=0;
	int ticks0=time(0);

	Star::seedStars();

	int wait_time=-1;
	int wait0 = time(0)-30;

	gameServer.gameStart();

	while (true) {
		NetworkConnection* new_client = gameServer.acceptNewClientConnection();
		if (new_client!=NULL) {
			string message = "Connected to server at "+string(SERVER_NAME)+".  Welcome!";
			//string message = "BETA TEST SERVER at "+string(SERVER_NAME)+".  Server may be taken down at any time!!!";
			(*new_client).queueMessage("MESG",message);
			gameServer.log(new_client,false,"BROA",message);
		}

		gameServer.pingClients();

		if (minutes15*15<=(time(0)-ticks0)/60) {
			stringstream message;
			message << "Server online for "<<(minutes15*15/60)<<"h"<<(minutes15*15%60)<<"m"<<".";
			gameServer.queueBroadcastMessage("BROA",message.str());
			minutes15++;
		}

		if (gameServer.isWait()) {
			wait0=time(0)+30;
			wait_time=6;
		}

		if (wait_time>(wait0-time(0))/5) {
			if (wait_time>=0) {
				stringstream message;
				message << (wait_time*5);
				gameServer.queueBroadcastMessage("WAIT",message.str());
				wait_time--;
			}
		}

		if ((wait0<time(0))&&(wait_time>=0)) {
			gameServer.gameStart();
			wait_time=-1;
		}


		gameServer.pushMessages();

		gameServer.handleMessages();
		gameServer.checkIfDead();

		//Sleep 50ms between loops.
#if defined(HAVE_NANOSLEEP)
		//Linux
		timespec tmReq;
		tmReq.tv_sec = 0;
		tmReq.tv_nsec = 50*1000 * 1000;

		(void)nanosleep(&tmReq, (timespec *)NULL);
#else
		//MingW
		_sleep(50);
#endif

	}
}

/*
 * Run game client.
 */
void startClient() {
	View view;
	Font font;
	SoundSystem soundSystem;
	ScreenSurface screenSurface;

	Ship ship;
	Ship::setMyShip(ship);
	Ship *my_ship = Ship::getMyShip();

	ShipSurface shipSurface;
	CargoSurface cargoSurface;
	MessageSurface messageSurface;
	OverviewSurface overviewSurface;
	Messages messages;
	AlienInterface alienInterface;
	GalaxySurface galaxySurface;
	RadarSurface radarSurface;
	AlienInterfaceSurface alienInterfaceSurface;
	InformationSurface informationSurface;

	MessageBuffer messageBuffer;
	bool typing = false;

   	Star::seedStars();

	MinimapSurface minimapSurface;

	NetworkConnection *networkConnection = NULL;

	SDL_WM_SetCaption("Galactic Artifact","Galactic Artifact");

	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

	SDL_EnableUNICODE(1);

	string *alien_types = Artifact::getAlienTypes();

	bool debug = false;
	bool quitToMenu = true;
	bool quitGame = false;

	MenuInterface menuInterface;
	MenuInterfaceSurface menuInterfaceSurface;

	while (!quitGame) {
		if (quitToMenu) {
			Artifact::setBetweenGames(false);
			Ship::removeAllShips();
			networkConnection = NetworkConnection::getClientNetworkConnection();
			if (networkConnection!=NULL) {
				//Close network connection in main game menu.
				(*networkConnection).queueMessage("PART");
				(*networkConnection).send();
#if defined(HAVE_NANOSLEEP)
				//Linux
				timespec tmReq;
				tmReq.tv_sec = 0;
				tmReq.tv_nsec = 100*1000 * 1000;

				(void)nanosleep(&tmReq, (timespec *)NULL);
#else
				//MingW
				_sleep(100);
#endif
			}
			//Reset EVERYTHING in main game menu.
			(*my_ship).reset();
			(*my_ship).resetCaptainName();
			Artifact::reset();
			Star::reset();
			View::reset();
			View::setUpdateOverride();
			Information::reset();
			Messages::reset();
			Messages::addMessageToQueue("");
			MenuInterface::reset();
			AlienInterface::reset();
			overviewSurface.reset();
			networkConnection = NULL;
			NetworkConnection::nullClientNetworkConnection();
		}
		while (quitToMenu) {
			soundSystem.playMusic();
			menuInterfaceSurface.blit();
			screenSurface.flip();

			KeyPressed keyPressed;
			menuInterface.keyPressed(keyPressed);
			if (menuInterface.isMenuExit()) {
				quitToMenu=false;
				break;
			} else if (keyPressed.getKeyPressed()==KEY_QUIT) {
				quitGame=true;
				break;
			}

			FrameTimer::delayFrame();
		}

		if (MenuInterface::getGameType()==-1) {
			quitGame=true;
		} else if (MenuInterface::getGameType()<2) {
			//SinglePlayer
			Ship::initShipLocation();
			Messages::addMessageToQueue("Welcome to Galactic Artifact!");
			Messages::addMessageToQueue("");
			FrameTimer::resetTimer();
		} else if (MenuInterface::getGameType()==2) {
			//Multiplayer
			networkConnection = NetworkConnection::getClientNetworkConnection();
			if (networkConnection==NULL) {
				NetworkConnection::initClient();
				networkConnection = NetworkConnection::getClientNetworkConnection();
				if (networkConnection==NULL) {
					Messages::addMessageToQueue("Unable to connect to server.  Switching to single player mode.",0xFFBFBFFF);
					Messages::addMessageToQueue("");
					MenuInterface::setGameType(0);
					Ship::initShipLocation();
				} else {
					Messages::addMessageToQueue("Connecting to server...",0xFFBFBFFF);
					stringstream version;
					version << GAME_VERSION;
					(*networkConnection).queueMessage("JOIN",version.str());
					Ship::nullShipLocation();
				}
			}
		}

		Star::findVisible();

		if (networkConnection==NULL) {
			Artifact::initPrizeLocation();
			Messages::addMessageToQueue("You are searching for '"+Artifact::getPrizeArtifact().getArtifactName()+"', a prized "+alien_types[Artifact::getPrizeArtifact().getArtifactType()]+" artifact!");
		}

		if (MenuInterface::getGameType()==1) {
			//Easy Mode
			for (int i=0;i<6;i++)
				(*my_ship).setUpgrades(i,10);
			(*my_ship).setFuel(400.0);
			(*my_ship).setFuelDist(400.0);
			(*my_ship).setCash(1000.0);
			(*my_ship).setCashDist(1000.0);
		}

		while ((!quitToMenu)&&(!quitGame)) {
			soundSystem.playMusic();
			networkConnection = NetworkConnection::getClientNetworkConnection();
			if (networkConnection!=NULL) {
				vector<string> messages = (*networkConnection).receive();
				if (messages.size()>0) {
					(*networkConnection).setLastRecvTime();
				}
				for (int i=0;i<(int)messages.size();i++) {
					NetworkEvent::handleMessage(messages[i]);
				}

				(*networkConnection).sendPing();
				(*networkConnection).send();
				if (((*networkConnection).getTimeSinceLastRecv()>15000)) {
					cout << "Time since last recv = "<<(*networkConnection).getTimeSinceLastRecv()<<endl;
					MenuInterface::setGameType(0);
					Ship::removeAllShips();
					networkConnection=NULL;
					Messages::addMessageToQueue("Lost Connection to server.  Switching to single player mode.",0xFFBFBFFF);
					NetworkConnection::nullClientNetworkConnection();
				}
			}

			if (debug) cout << "Begining Loop " << FrameTimer::getInterimFrameLength() ;
			shipSurface.blit();
			cargoSurface.blit();
			messageSurface.blit();
			overviewSurface.blit();

			if (debug) cout << " Basic Surfaces " << FrameTimer::getInterimFrameLength() ;
			galaxySurface.blit();

			if (debug) cout << " Galaxy Surface " << FrameTimer::getInterimFrameLength();
			minimapSurface.blit();
			View::resetOverride();

			if ((*my_ship).isVisitingAliens()) {
				alienInterfaceSurface.blit();
			} else {
				radarSurface.blit();
				if (debug) cout << " Radar Surface " << FrameTimer::getInterimFrameLength();
				View::resetOverride();
			}

			screenSurface.flip();
			if (debug) cout << " Flip Screen " << FrameTimer::getInterimFrameLength();
			if (typing) {
				messageBuffer.getKey();
				if (messageBuffer.isQuit()) {
					quitGame=true;
				} else if (messageBuffer.isMessageComplete()) {
					string message = messageBuffer.getMessage();
					if ((message!="") && (networkConnection!=NULL)) {
						(*networkConnection).queueMessage("TALK",message);
					}
					messageBuffer.clearMessage();
					typing = false;
				}
			} else {
				KeyPressed keyPressed;

				if (keyPressed.getKeyPressed()==KEY_TYPE) {
					typing = true;
				}

				if ((keyPressed.getKeyPressed()==KEY_CHEAT1)&&(keyPressed.isShiftPressed())) {
					(*my_ship).beckyCheat();
				}

				if (keyPressed.getKeyPressed()==KEY_MUTE) {
					if (soundSystem.mute())
						Messages::addMessageToQueue("Sound Muted.");
					else
						Messages::addMessageToQueue("Sound Unmuted.");
				}

				if (keyPressed.getKeyPressed()==KEY_CHEAT2) {
					galaxySurface.toggleArtifactCursorCheat();
				}

				if (keyPressed.getKeyPressed()==KEY_CHEAT3) {
					debug = !debug;
				}

				if ((*my_ship).isVisitingAliens()) {
					alienInterface.keyPressed(keyPressed);
				} else {
					view.keyPressed(keyPressed);
					view.mousePressed(keyPressed);
				}

				if (keyPressed.getKeyPressed()==KEY_PAUSE) {
					if (networkConnection==NULL) {
						FrameTimer::pauseTimer();
						while (true) {
							soundSystem.playMusic();
							keyPressed = KeyPressed();
								if (keyPressed.getKeyPressed()==KEY_PAUSE)
									break;
							SDL_Delay(100);
						}
						FrameTimer::startTimer();
					} else {
						Messages::addMessageToQueue("Pause not available in Network Game.");
					}
				}

				if (keyPressed.getKeyPressed()==KEY_QUIT) {
					quitGame=true;
				}

				if (keyPressed.getKeyPressed()==KEY_ESC) {
					SoundSystem::stopSound(-1);
					SoundSystem::resumeMusic();
					quitToMenu=true;
				}
			}
			if (debug) cout << " Before Delay " << FrameTimer::getInterimFrameLength() << endl;


			if (quitToMenu) {
				//"Are you sure?" code
				VerifyQuitSurface vqs;
				vqs.blit();
				screenSurface.flip();
				while (true) {
					KeyPressed keyPressed;
					if (keyPressed.getKeyPressed()==KEY_CONFIRM) {
						break;
					} else if (keyPressed.getKeyPressed()==KEY_UNPRESSED) {
						//Do Nothing.
					} else  {
						quitToMenu=false;
						break;
					}
					FrameTimer::delayFrame();
				}
			}

			FrameTimer::delayFrame();

			View::incrementTime();
			Ship::moveShips();

			if (Ship::isVictory()||Ship::isFailure())
				break;
		}

		VictorySurface victorySurface(Ship::isVictory());

		SoundSystem::stopSound(-1);
		SoundSystem::resumeMusic();

		if (Ship::isVictory()||Ship::isFailure()) {
			Artifact::setBetweenGames(true);
			if (Ship::isVictory())
				if (networkConnection!=NULL)
					(*networkConnection).queueMessage("VICT");
			Ship::resetVictoryFailure();

			//Move this here, as a KICK message may set gametype to 0, but we don't want
			//to quit just yet.
			if (MenuInterface::getGameType()<2)
				quitToMenu=true;

			while (true) {
				soundSystem.playMusic();
				galaxySurface.blit();
				messageSurface.blit();
				minimapSurface.blit();
				victorySurface.blit();
				screenSurface.flip();

				if (networkConnection!=NULL) {
					vector<string> messages = (*networkConnection).receive();
					if (messages.size()>0) {
						(*networkConnection).setLastRecvTime();
					}
					for (int i=0;i<(int)messages.size();i++) {
						NetworkEvent::handleMessage(messages[i]);
					}

					(*networkConnection).sendPing();

					(*networkConnection).send();
					if (((*networkConnection).getTimeSinceLastRecv()>15000)) {
						cout << "Time since last recv = "<<(*networkConnection).getTimeSinceLastRecv()<<endl;
						MenuInterface::setGameType(0);
						Ship::removeAllShips();
						networkConnection=NULL;
						Messages::addMessageToQueue("Lost Connection to server.  Switching to single player mode.",0xFFBFBFFF);
						NetworkConnection::nullClientNetworkConnection();
					}

				}

				if (!Artifact::isBetweenGames())
					//Get off this screen and get playing!
					//Also protects us from font changing.
					break;

				if (typing) {
					messageBuffer.getKey();
					if (messageBuffer.isQuit()) {
						quitGame=true;
					} else if (messageBuffer.isMessageComplete()) {
						string message = messageBuffer.getMessage();
						if ((message!="") && (networkConnection!=NULL)) {
							(*networkConnection).queueMessage("TALK",message);
						}
						messageBuffer.clearMessage();
						typing = false;
					}
				} else {
					KeyPressed keyPressed;

					if (keyPressed.getKeyPressed()==KEY_TYPE) {
						typing = true;
					}

					if ((keyPressed.getKeyPressed()==KEY_CHEAT1)&&(keyPressed.isShiftPressed())) {
						(*my_ship).beckyCheat();
					}

					if (keyPressed.getKeyPressed()==KEY_MUTE) {
						if (soundSystem.mute())
							Messages::addMessageToQueue("Sound Muted.");
						else
							Messages::addMessageToQueue("Sound Unmuted.");
					}

					if (keyPressed.getKeyPressed()==KEY_CHEAT2) {
						galaxySurface.toggleArtifactCursorCheat();
					}

					if (keyPressed.getKeyPressed()==KEY_CHEAT3) {
						debug = !debug;
					}

					if (keyPressed.getKeyPressed()==KEY_PAUSE) {
						if (networkConnection==NULL) {
							FrameTimer::pauseTimer();
							while (true) {
								soundSystem.playMusic();
								keyPressed = KeyPressed();
									if (keyPressed.getKeyPressed()==KEY_PAUSE)
										break;
								SDL_Delay(100);
							}
							FrameTimer::startTimer();
						} else {
							Messages::addMessageToQueue("Pause not available in Network Game.");
						}
					}

					if (keyPressed.getKeyPressed()==KEY_QUIT) {
						quitGame=true;
						break;
					}

					if (keyPressed.getKeyPressed()==KEY_ESC) {
						//"Are you sure?" code
						VerifyQuitSurface vqs;
						vqs.blit();
						screenSurface.flip();
						bool realQuit = false;
						while (true) {
							KeyPressed keyPressed;
							if (keyPressed.getKeyPressed()==KEY_CONFIRM) {
								realQuit=true;
								quitToMenu=true;
								break;
							} else if (keyPressed.getKeyPressed()==KEY_UNPRESSED) {
								//Do Nothing.
							} else  {
								break;
							}
							FrameTimer::delayFrame();
						}
						if (realQuit)
							break;
					}


					if ((keyPressed.getKeyPressed()==KEY_ENTER)||(keyPressed.isMousePressed())) {
						if (MenuInterface::getGameType()<2)
							quitToMenu=true;
						break;
					}
				}


				FrameTimer::delayFrame();
				View::incrementTime();
				Ship::moveShips();
			}
		}

		if (quitGame)
			break;

		if ((quitToMenu)&&(networkConnection!=NULL)) {
			(*networkConnection).queueMessage("PART");
			(*networkConnection).send();
#if defined(HAVE_NANOSLEEP)
			//Linux
			timespec tmReq;
			tmReq.tv_sec = 0;
			tmReq.tv_nsec = 100*1000 * 1000;

			(void)nanosleep(&tmReq, (timespec *)NULL);
#else
			//MingW
			_sleep(100);
#endif
			networkConnection = NULL;
			NetworkConnection::nullClientNetworkConnection();
		}
	}

	//FIXME: Other quit routines should be here.
	SDL_Quit();
}

/*
 * Main program.  Detect --server command line flag and run client or server appropriately.
 */
int main(int argc, char *argv[]) {
	if (argc>1) {
		stringstream arg1;
		arg1 << argv[1];
		if (arg1.str()=="--server") {
			startServer();
			return 0;
		}
	}
	startClient();
	return 0;
}
