/*
 * GalaxySurface
 *
 * Renders the galaxy of stars and the background halo image onto the screen surface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"
//#include "SDL/SDL_rotozoom.h"
#include "SDL/SDL_image.h"
#include "SDL/sprig.h"
#include "SDL/sprig_inline.h"

#include "Definitions.h"
#include "NumberGenerator.h"
#include "Font.h"
#include "Ship.h"
#include "View.h"
#include "Coords.h"
#include "Artifact.h"
#include "Star.h"
#include "FrameTimer.h"
#include "InformationSurface.h"
#include "ScreenSurface.h"
#include "GalaxySurface.h"

using namespace std;


SDL_Surface *GalaxySurface::_surface;
SDL_Surface *GalaxySurface::_ship_surface;
SDL_Surface *GalaxySurface::_cursor_surface;
SDL_Surface *GalaxySurface::_artifact_cursor_surface;
SDL_Rect GalaxySurface::_rect;
SDL_Rect GalaxySurface::_cursor_rect;
SDL_Rect GalaxySurface::_artifact_cursor_rect;
SDL_Surface *GalaxySurface::_galaxy_surface_1;
SDL_Surface *GalaxySurface::_galaxy_surface_2;
SDL_Surface *GalaxySurface::_galaxy_surface_3;
SDL_Surface *GalaxySurface::_galaxy_surface_4;
bool GalaxySurface::_artifact_cursor_cheat;

/*
 * Constructor.  Initialize all Galaxy surfaces.  Must be called before blit()
 */
GalaxySurface::GalaxySurface() {
	_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2, SCREEN_HEIGHT, SCREEN_DEPTH,0,0,0,0);
	_ship_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, CURSOR_SIZE*6, CURSOR_SIZE, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(_ship_surface, SDL_SRCCOLORKEY, 0x00000000);
	_cursor_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, CURSOR_SIZE, CURSOR_SIZE, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(_cursor_surface, SDL_SRCCOLORKEY, 0x00000000);

	_artifact_cursor_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, CURSOR_SIZE, CURSOR_SIZE, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(_artifact_cursor_surface, SDL_SRCCOLORKEY, 0x00000000);


	_rect.x=0;
	_rect.y=0;
	_rect.w=SCREEN_CENTER*2;
	_rect.h=SCREEN_HEIGHT;

	_cursor_rect.x=0;
	_cursor_rect.y=0;
	_cursor_rect.w=CURSOR_SIZE;
	_cursor_rect.h=CURSOR_SIZE;

	_artifact_cursor_rect.x=0;
	_artifact_cursor_rect.y=0;
	_artifact_cursor_rect.w=CURSOR_SIZE;
	_artifact_cursor_rect.h=CURSOR_SIZE;

	_galaxy_surface_1 = IMG_Load("./Sheets/Galaxy480.jpg");
	_galaxy_surface_2 = IMG_Load("./Sheets/Galaxy1440.jpg");
	_galaxy_surface_3 = IMG_Load("./Sheets/Galaxy4320.jpg");
	_galaxy_surface_4 = IMG_Load("./Sheets/Galaxy12960s.jpg");

	_artifact_cursor_cheat=false;

}

/*
 * Destructor.  Free all surfaces.
 */
GalaxySurface::~GalaxySurface() {
	SDL_FreeSurface(_surface);
	SDL_FreeSurface(_ship_surface);
	SDL_FreeSurface(_cursor_surface);
	SDL_FreeSurface(_artifact_cursor_surface);
	SDL_FreeSurface(_galaxy_surface_1);
	SDL_FreeSurface(_galaxy_surface_2);
	SDL_FreeSurface(_galaxy_surface_3);
	SDL_FreeSurface(_galaxy_surface_4);
}

/*
 * Blit the galaxy surface (halo, stars, ships, cursor, artifact cursor) to the screen surface.
 */
void GalaxySurface::blit() {
	bool debug = false;
	if (View::isUpdated()||View::isUpdateOverride()) {
		int x_pos,y_pos;
		float center_x,center_y;
		int x_line_0,x_line_1,x_line_2,x_line_3;
		int y_line_0,y_line_1,y_line_2,y_line_3;

		Font::setFont("default");

		if (debug) cout << "Galaxy preBox " << FrameTimer::getInterimFrameLength() << endl;

		boxColor(_surface,0,0,SCREEN_WIDTH,SCREEN_HEIGHT,0x000000FF);
		//filledCircleColor(_surface,SCREEN_CENTER,SCREEN_CENTER,SCREEN_CENTER,0x424C58FF);

		if (debug) cout << "Galaxy postBox " << FrameTimer::getInterimFrameLength() << endl;

		if (!View::isZoomedIn()) {
			Star::findVisible();
		}

		if (debug) cout << "Galaxy postVis " << FrameTimer::getInterimFrameLength() << endl;

		vector<Star> visible_stars = Star::getVisibleStars();
		float scale = View::getScale();

		Star star;
		Coords delta,offset;
		int size,color;
		offset = View::getOffset();
		int level = View::getLevel();

		if (debug) cout << "Galaxy PreSurf " << FrameTimer::getInterimFrameLength() << endl;

		if ((level==0)||(level==1)) {
			int width = (int)(480*scale/100000.0);
			SDL_Surface *source = SDL_CreateRGBSurface(SDL_HWSURFACE, width,width, 16,0,0,0,0);
			Coords xy = offset-Coords(scale/2,scale/2);
			SDL_Rect source_rect;
			source_rect.w=source_rect.h=width;
			source_rect.x=(Sint32)((xy.getX()+50000)*SCREEN_CENTER*2/100000.0);
			source_rect.y=(Sint32)((xy.getY()+50000)*SCREEN_CENTER*2/100000.0);
			SDL_BlitSurface (_galaxy_surface_1,&source_rect,source,NULL);

			//SDL_Surface *zoomed = zoomSurface (source, 100000.0/scale, 100000.0/scale, 0);
			SDL_Surface *zoomed = SPG_Scale (source, 100000.0/scale, 100000.0/scale);
			SDL_BlitSurface(zoomed,NULL,_surface,NULL);
			SDL_FreeSurface(source);
			SDL_FreeSurface(zoomed);
		}

		if (debug) cout << "Galaxy SurfA " << FrameTimer::getInterimFrameLength() << endl;

		if ((level==1)||(level==2)) {
			int width = (int)(SCREEN_CENTER*2*scale/33333.333);
			if (level==1)
				width=SCREEN_CENTER*2;
			SDL_Surface *source = SDL_CreateRGBSurface(SDL_HWSURFACE, width,width, 16,0,0,0,0);
			float z_scale=100000/3.0;
			if (level==2)
				z_scale=scale;
			Coords xy;
			if (level==1)
				xy = View::getOffsetDest()-Coords(z_scale/2,z_scale/2);
			else
				xy=offset-Coords(z_scale/2,z_scale/2);
			SDL_Rect source_rect;
			source_rect.w=source_rect.h=width;
			source_rect.x=(Sint32)((xy.getX()+50000)*SCREEN_CENTER*6/100000);
			source_rect.y=(Sint32)((xy.getY()+50000)*SCREEN_CENTER*6/100000);
			SDL_BlitSurface (_galaxy_surface_2,&source_rect,source,NULL);

			SDL_Rect target_rect;
			if (level==1) {
				int x_pos,y_pos;
				Coords delta = xy-offset;
				delta.getIntValues(&x_pos,&y_pos,scale);
				target_rect.x = x_pos;
				target_rect.y = y_pos;
			} else {
				target_rect.x = 0;
				target_rect.y = 0;
			}
			//SDL_Surface *zoomed = zoomSurface (source, 33333.333/scale, 33333.333/scale, 0);
			SDL_Surface *zoomed = SPG_Scale (source, 33333.333/scale, 33333.333/scale);
			SDL_BlitSurface(zoomed,NULL,_surface,&target_rect);
			SDL_FreeSurface(source);
			SDL_FreeSurface(zoomed);
		}

		if (debug) cout << "Galaxy SurfB " << FrameTimer::getInterimFrameLength() << endl;

		if ((level==2)||(level==3)) {
			int width = (int)(SCREEN_CENTER*2*scale/11111.111);
			if (level==2)
				width=SCREEN_CENTER*2;
			SDL_Surface *source = SDL_CreateRGBSurface(SDL_HWSURFACE, width,width, 16,0,0,0,0);
			float z_scale=100000/9.0;
			if (level==3)
				z_scale=scale;
			Coords xy;
			if (level==2)
				xy = View::getOffsetDest()-Coords(z_scale/2,z_scale/2);
			else
				xy=offset-Coords(z_scale/2,z_scale/2);
			SDL_Rect source_rect;
			source_rect.w=source_rect.h=width;
			source_rect.x=(Sint32)((xy.getX()+50000)*SCREEN_CENTER*18/100000);
			source_rect.y=(Sint32)((xy.getY()+50000)*SCREEN_CENTER*18/100000);
			SDL_BlitSurface (_galaxy_surface_3,&source_rect,source,NULL);

			SDL_Rect target_rect;
			if (level==2) {
				int x_pos,y_pos;
				Coords delta = xy-offset;
				delta.getIntValues(&x_pos,&y_pos,scale);
				target_rect.x = x_pos;
				target_rect.y = y_pos;
			} else {
				target_rect.x = 0;
				target_rect.y = 0;
			}
			//SDL_Surface *zoomed = zoomSurface (source, 11111.11/scale, 11111.111/scale, 0);
			SDL_Surface *zoomed = SPG_Scale (source, 11111.11/scale, 11111.111/scale);

			SDL_BlitSurface(zoomed,NULL,_surface,&target_rect);
			SDL_FreeSurface(source);
			SDL_FreeSurface(zoomed);
		}

		if (debug) cout << "Galaxy SurfC " << FrameTimer::getInterimFrameLength() << endl;

		if (level==3) {
			int width = SCREEN_CENTER*2;
			float z_scale=100000/27.0;
			Coords xy;
			if (level==3)
				xy = View::getOffsetDest()-Coords(z_scale/2,z_scale/2);
			else
				xy=offset-Coords(z_scale/2,z_scale/2);
			SDL_Rect source_rect;
			source_rect.w=source_rect.h=width;
			source_rect.x=(Sint32)((xy.getX()+50000)*SCREEN_CENTER*54/100000)-5670;
			source_rect.y=(Sint32)((xy.getY()+50000)*SCREEN_CENTER*54/100000)-5670;
			if ((source_rect.x>=0) && (source_rect.x<SCREEN_CENTER*6) &&
				(source_rect.y>=0) && (source_rect.y<SCREEN_CENTER*6)) {

				SDL_Surface *source = SDL_CreateRGBSurface(SDL_HWSURFACE, width,width, 16,0,0,0,0);

				SDL_BlitSurface (_galaxy_surface_4,&source_rect,source,NULL);

				SDL_Rect target_rect;
				if (level==3) {
					int x_pos,y_pos;
					Coords delta = xy-offset;
					delta.getIntValues(&x_pos,&y_pos,scale);
					target_rect.x = x_pos;
					target_rect.y = y_pos;
				} else {
					target_rect.x = 0;
					target_rect.y = 0;
				}
				//SDL_Surface *zoomed = zoomSurface (source, 3703.7037/scale, 3703.7037/scale, 0);
				SDL_Surface *zoomed = SPG_Scale (source, 3703.7037/scale, 3703.7037/scale);

				SDL_BlitSurface(zoomed,NULL,_surface,&target_rect);
				SDL_FreeSurface(source);
				SDL_FreeSurface(zoomed);
			}
		}

		if (debug) cout << "Galaxy Gather Visible " << FrameTimer::getInterimFrameLength() << endl;


		for (int i=visible_stars.size()-1;i>=0;i--) {
			star = visible_stars[i];
			delta = star.getPosition()-offset;
			delta.getIntValues(&x_pos,&y_pos,scale);
			size = star.getSize();
			color = star.getStarColor();

			filledCircleColor(_surface,x_pos,y_pos,size,color);
		}

		if (debug) cout << "Galaxy Draw Stars " << FrameTimer::getInterimFrameLength() << endl;

		center_x=center_y=0;

		int *location = View::getLocation();

		for (int i=0;i<=level;i++) {
			float this_scale = 100000.0/pow(3,i);

			if ((i>0)&&((location[i-1]==1)||(location[i-1]==2)||(location[i-1]==3)))
				center_y += this_scale;
			if ((i>0)&&((location[i-1]==7)||(location[i-1]==8)||(location[i-1]==9)))
				center_y -= this_scale;
			if ((i>0)&&((location[i-1]==1)||(location[i-1]==4)||(location[i-1]==7)))
				center_x -= this_scale;
			if ((i>0)&&((location[i-1]==3)||(location[i-1]==6)||(location[i-1]==9)))
				center_x += this_scale;

			if (i<level-2)
				continue;

			x_line_0 = SCREEN_CENTER+(int)((center_x-this_scale/2.0-offset.getX())*2.0*SCREEN_CENTER/scale);

			x_line_1 = SCREEN_CENTER+(int)((center_x-this_scale/6.0-offset.getX())*2.0*SCREEN_CENTER/scale);
			x_line_2 = SCREEN_CENTER+(int)((center_x+this_scale/6.0-offset.getX())*2.0*SCREEN_CENTER/scale);
			x_line_3 = SCREEN_CENTER+(int)((center_x+this_scale/2.0-offset.getX())*2.0*SCREEN_CENTER/scale);

			y_line_0 = SCREEN_CENTER+(int)((center_y-this_scale/2.0-offset.getY())*2.0*SCREEN_CENTER/scale);
			y_line_1 = SCREEN_CENTER+(int)((center_y-this_scale/6.0-offset.getY())*2.0*SCREEN_CENTER/scale);
			y_line_2 = SCREEN_CENTER+(int)((center_y+this_scale/6.0-offset.getY())*2.0*SCREEN_CENTER/scale);
			y_line_3 = SCREEN_CENTER+(int)((center_y+this_scale/2.0-offset.getY())*2.0*SCREEN_CENTER/scale);

			//Reduce effects of grid doubling at borders
			if (i<7) {
				if ((x_line_0>1) && (x_line_0<SCREEN_CENTER*2-2))
					vlineColor(_surface,x_line_0,y_line_0,y_line_3,0x3F3FFFFF);
				if ((x_line_1>1) && (x_line_1<SCREEN_CENTER*2-2))
					vlineColor(_surface,x_line_1,y_line_0,y_line_3,0x3F3FFFFF);
				if ((x_line_2>1) && (x_line_2<SCREEN_CENTER*2-2))
					vlineColor(_surface,x_line_2,y_line_0,y_line_3,0x3F3FFFFF);
				if ((x_line_3>1) && (x_line_3<SCREEN_CENTER*2-2))
					vlineColor(_surface,x_line_3,y_line_0,y_line_3,0x3F3FFFFF);

				if ((y_line_0>1) && (y_line_0<SCREEN_CENTER*2-2))
					hlineColor(_surface,x_line_0,x_line_3,y_line_0,0x3F3FFFFF);
				if ((y_line_1>1) && (y_line_1<SCREEN_CENTER*2-2))
					hlineColor(_surface,x_line_0,x_line_3,y_line_1,0x3F3FFFFF);
				if ((y_line_2>1) && (y_line_2<SCREEN_CENTER*2-2))
					hlineColor(_surface,x_line_0,x_line_3,y_line_2,0x3F3FFFFF);
				if ((y_line_3>1) && (y_line_3<SCREEN_CENTER*2-2))
					hlineColor(_surface,x_line_0,x_line_3,y_line_3,0x3F3FFFFF);
			}

			if ((i==level)&&(x_line_0<SCREEN_CENTER/2)&&(i<7)) {
				characterColor(_surface,x_line_1-10,y_line_3-10,'1',0xFFFFFFFF);
				characterColor(_surface,x_line_2-10,y_line_3-10,'2',0xFFFFFFFF);
				characterColor(_surface,x_line_3-10,y_line_3-10,'3',0xFFFFFFFF);
				characterColor(_surface,x_line_1-10,y_line_2-10,'4',0xFFFFFFFF);
				characterColor(_surface,x_line_2-10,y_line_2-10,'5',0xFFFFFFFF);
				characterColor(_surface,x_line_3-10,y_line_2-10,'6',0xFFFFFFFF);
				characterColor(_surface,x_line_1-10,y_line_1-10,'7',0xFFFFFFFF);
				characterColor(_surface,x_line_2-10,y_line_1-10,'8',0xFFFFFFFF);
				characterColor(_surface,x_line_3-10,y_line_1-10,'9',0xFFFFFFFF);
			}

		}

		if (debug) cout << "Galaxy Draw Lines " << FrameTimer::getInterimFrameLength() << endl;
	}

	InformationSurface::blit();
	if (debug) cout << " Information Surface " << FrameTimer::getInterimFrameLength()<<endl;

	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);

	if (debug) cout << "Galaxy Blit " << FrameTimer::getInterimFrameLength() << endl;

	if ((View::getLevel()==7) && (View::getScale()==View::getScaleDest()))
		SDL_BlitSurface(_surface,&_cursor_rect,ScreenSurface::getScreenSurface(),&_cursor_rect);

	if (debug) cout << "Galaxy Cursor Pre-blit " << FrameTimer::getInterimFrameLength() << endl;


	blitShip();
	if ((View::getLevel()==7) && (View::getScale()==View::getScaleDest()))
		blitCursor();

	if (debug) cout << "Galaxy Cursor Post-blit " << FrameTimer::getInterimFrameLength() << endl;
	if (_artifact_cursor_cheat||Artifact::isBetweenGames())
		blitArtifactCursor();
}

/*
 * Draw the player ship cursor to the galaxy surface.
 */
void GalaxySurface::blitShip() {
	map<int,Ship>::iterator iter;
	map<const int, Ship, map_int_compare> *ships = Ship::getShips();
	for( iter = (*ships).begin(); iter!=(*ships).end(); ++iter ) {
		Ship other_ship = (*ships)[iter->first];
		blitShip(other_ship);
	}

	Ship *my_ship = Ship::getMyShip();
	blitShip(*my_ship);
}

/*
 * Draw some other ship to the galaxy surface (for multiplayer)
 */
void GalaxySurface::blitShip(Ship my_ship) {
	Coords delta = my_ship.getShipCoords()-View::getOffset();
	int x_pos,y_pos;
	delta.getIntValues(&x_pos,&y_pos,View::getScale());

	Sint32 ship_color = 0xFF3F3FFF;
	ship_color = my_ship.getShipColor();

	if (!((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)))
		return;

	int i=90+(-SDL_GetTicks()/5)%360;

	Font::setFont("default");

	boxColor(_ship_surface,0,0,CURSOR_SIZE*6,CURSOR_SIZE,0x000000FF);

	filledPieColor(_ship_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,8,270+i,360+i,ship_color);
	filledPieColor(_ship_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,8,90+i,180+i,ship_color);

	filledCircleColor(_ship_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,5,0x000000FF);

	stringColor(_ship_surface,CURSOR_SIZE,0,my_ship.getCaptainName().c_str(),ship_color);

	SDL_Rect location_rect;
	location_rect.x = x_pos-CURSOR_SIZE/2;
	location_rect.y = y_pos-CURSOR_SIZE/2;
	location_rect.h = CURSOR_SIZE;
	location_rect.w = CURSOR_SIZE*6;

	SDL_Rect source_rect;
	source_rect.x=source_rect.y=0;
	source_rect.h = CURSOR_SIZE;
	source_rect.w = CURSOR_SIZE*6;

	//Ensure cursor doesn't get pushed down or right if at edge of screen
	if (location_rect.x<0) {
		source_rect.x-=location_rect.x;
		source_rect.w+=location_rect.x;
		location_rect.x=0;
	}

	if (location_rect.y<0) {
		source_rect.y-=location_rect.y;
		source_rect.h+=location_rect.y;
		location_rect.y=0;
	}

	if (location_rect.x+location_rect.w>=SCREEN_CENTER*2) {
		source_rect.w=SCREEN_CENTER*2-location_rect.x;
	}

	if (location_rect.y+location_rect.h>=SCREEN_CENTER*2) {
		source_rect.h=SCREEN_CENTER*2-location_rect.y;
	}

	SDL_BlitSurface(_ship_surface,&source_rect,ScreenSurface::getScreenSurface(),&location_rect);

}

/*
 * Blit green User Interface cursor on galaxy surface.
 */
bool GalaxySurface::blitCursor() {
	Coords delta = View::getCursorCoords()-View::getOffset();
	int x_pos,y_pos;
	delta.getIntValues(&x_pos,&y_pos,View::getScale());

	if (!((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)))
		return false;

	int i=(-SDL_GetTicks()/5)%360;

	boxColor(_cursor_surface,0,0,CURSOR_SIZE,CURSOR_SIZE,0x000000FF);

	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,8,270+i,360+i,0x3FFF3FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,8,90+i,180+i,0x3FFF3FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,6,270+i,3600+i,0x3FFF3FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,6,90+i,180+i,0x3FFF3FFF);

	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,9,270+i,360+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,9,90+i,180+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,7,270+i,360+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,7,90+i,180+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,5,270+i,360+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,5,90+i,180+i,0x0F0F0FFF);

	SDL_Rect location_rect;
	location_rect.x = x_pos-CURSOR_SIZE/2;
	location_rect.y = y_pos-CURSOR_SIZE/2;
	location_rect.w = location_rect.h = CURSOR_SIZE;

	SDL_Rect source_rect;
	source_rect.x=source_rect.y=0;
	source_rect.w = source_rect.h = CURSOR_SIZE;

	//Ensure cursor doesn't get pushed down or right if at edge of screen
	if (location_rect.x<0) {
		source_rect.x-=location_rect.x;
		source_rect.w+=location_rect.x;
		location_rect.x=0;
	}

	if (location_rect.y<0) {
		source_rect.y-=location_rect.y;
		source_rect.h+=location_rect.y;
		location_rect.y=0;
	}

	if (location_rect.x+location_rect.w>=SCREEN_CENTER*2) {
		source_rect.w=SCREEN_CENTER*2-location_rect.x;
	}

	if (location_rect.y+location_rect.h>=SCREEN_CENTER*2) {
		source_rect.h=SCREEN_CENTER*2-location_rect.y;
	}

	SDL_BlitSurface(_cursor_surface,&source_rect,ScreenSurface::getScreenSurface(),&location_rect);

	_cursor_rect = location_rect;

	return true;
}

/*
 * Blit artifact cursor on screen surface.  Used at the end of multiplayer games.
 * Also used by testers, and dirty cheaters in real games.
 */
bool GalaxySurface::blitArtifactCursor() {
	Coords delta = Artifact::getPrizeStar().getPosition()-View::getOffset();
	int x_pos,y_pos;
	delta.getIntValues(&x_pos,&y_pos,View::getScale());

	if (!((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)))
		return false;

	int i=(-SDL_GetTicks()/5)%360;

	boxColor(_artifact_cursor_surface,0,0,CURSOR_SIZE,CURSOR_SIZE,0x000000FF);

	filledPieColor(_artifact_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,6,270+i,360+i,0xFFFFFFFF);
	filledPieColor(_artifact_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,6,90+i,180+i,0xFFFFFFFF);

	SDL_Rect location_rect;
	location_rect.x = x_pos-CURSOR_SIZE/2;
	location_rect.y = y_pos-CURSOR_SIZE/2;
	location_rect.w = location_rect.h = CURSOR_SIZE;

	SDL_Rect source_rect;
	source_rect.x=source_rect.y=0;
	source_rect.w = source_rect.h = CURSOR_SIZE;

	//Ensure cursor doesn't get pushed down or right if at edge of screen
	if (location_rect.x<0) {
		source_rect.x-=location_rect.x;
		source_rect.w+=location_rect.x;
		location_rect.x=0;
	}

	if (location_rect.y<0) {
		source_rect.y-=location_rect.y;
		source_rect.h+=location_rect.y;
		location_rect.y=0;
	}

	if (location_rect.x+location_rect.w>=SCREEN_CENTER*2) {
		source_rect.w=SCREEN_CENTER*2-location_rect.x;
	}

	if (location_rect.y+location_rect.h>=SCREEN_CENTER*2) {
		source_rect.h=SCREEN_CENTER*2-location_rect.y;
	}

	SDL_BlitSurface(_artifact_cursor_surface,&source_rect,ScreenSurface::getScreenSurface(),&location_rect);

	_artifact_cursor_rect = location_rect;

	return true;
}

/*
 * Turn off Artifact even during regular gameplay.  THIS IS CHEATING!
 */
void GalaxySurface::toggleArtifactCursorCheat() {
	_artifact_cursor_cheat=!_artifact_cursor_cheat;
}

/*
 * See if Artifact Cursor Cheat is on.
 */
bool GalaxySurface::getArtifactCursorCheat() {
	return _artifact_cursor_cheat;
}

/*
 * Return the Galaxy Surface (so that Information Surface can paint directly to it).
 */
SDL_Surface *GalaxySurface::getGalaxySurface() {
	return _surface;
}
