/*
 * GameServer
 *
 * Server backend that handles network events from clients, creating NetworkConnections and
 * exchanges messages between players.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef GAME_SERVER_H
#define GAME_SERVER_H

#include <string>
#include <vector>
#include "SDL/SDL.h"
#include "SDL/SDL_net.h"
#include "NetworkConnection.h"

class GameServer {
private:
	std::string _hostname;
	int _port;
	IPaddress _ip_address;
	TCPsocket _server_socket;
	std::vector<NetworkConnection> _clients;
	std::vector<std::string> _plundered_stars;
	bool _wait;
public:
	GameServer();
	~GameServer();
	NetworkConnection initClient();
	bool initHost();
	NetworkConnection *acceptNewClientConnection();
	void queueBroadcastMessage(std::string message);
	void queueBroadcastMessage(std::string message,std::string command);
	void queueBroadcastMessageExcept(int except,std::string message);
	void queueBroadcastMessageExcept(int except,std::string message,std::string command);
	void pushMessages();
	void handleMessages();
	bool isWait();
	void gameStart();
	std::string getTime();
	std::string pad(std::string s, int length, char with);
	void log(NetworkConnection *client, bool inbound, string command, string message);
	void kickInactive();
	void checkIfDead();
	void pingClients();
};

#endif
