/*
 * Information
 *
 * Object describing a single piece of alien information.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef INFORMATION_H
#define INFORMATION_H

#include <vector>

#include "Coords.h"

#define CIRCLE 0
#define INV_CIRCLE 1
#define INV_WEDGE 3
#define WEDGE 4
#define SECTOR 6
#define INV_SECTOR 7

class Information {
private:
	int _type;
	Coords _position;
	float _value1;
	float _value2;
	static std::vector<Information> _information;
	static Information _last;
public:
	Information();
	Information(const Information &b);
	Information(int type, float value1, float value2);
	Information(int type, float value1);
	static void addInformation (Information i);
	static void removeInformation ();
	static void readdInformation ();
	static void promoteInformation (int i);
	static std::vector<Information> getInformation ();
	int getType();
	float getValue1();
	float getValue2();
	Coords getPosition();
	bool isExcludedBy(Information info);
	static void reset();
};

#endif
