/*
 * MenuInterface
 *
 * Back end engine for the main game menu.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include <sstream>
#include <string>

#include "KeyPressed.h"
#include "Information.h"
#include "MenuInterface.h"


using namespace std;

int MenuInterface::_page;
int MenuInterface::_vpos;
int MenuInterface::_selected_option;
bool MenuInterface::_new_page;
bool MenuInterface::_menu_exit;
int MenuInterface::_game_type;

/*
 * Constructor.
 */
MenuInterface::MenuInterface() {
	_page=0;
	_selected_option=0;
	_new_page=true;
	_menu_exit=false;
	_game_type=0;
}

/*
 * Takes in a KeyPressed object and takes action on the MenuInterface.
 */
void MenuInterface::keyPressed(KeyPressed kp) {
	stringstream ss;
	ss.precision(0);

	int keypad = kp.getKeyPressed();
	int mouse_pressed = kp.isMousePressed();

	if ((keypad==KEY_ENTER)||(mouse_pressed==1)) {
		_new_page=true;

		if (_page==0) {
			_page++;
		} else if (_page==1) {
			switch (_selected_option) {
				case 0:
					//Start Network Game
					_menu_exit=true;
					_game_type=2;
					break;
				case 1:
					//Start Local Game
					_menu_exit=true;
					_game_type=0;
					break;
				case 2:
					//Start Local Game (Easy)
					_menu_exit=true;
					_game_type=1;
					break;
				case 3:
					//Instructions
					_page=10;
					break;
				case 4:
					//Options
					_page=20;
					break;
				case 5:
					//Achievements
					_page=30;
					break;
				case 6:
					//Credits
					_page=40;
					break;
				default:
					//Exit
					_menu_exit=true;
					_game_type=-1;
					break;
			}
		} else {
			_page=1;
		}
		_selected_option=0;
	} else if ((keypad==KEY_8)||(mouse_pressed==4)) {
		//Up
		vector<string> options = MenuInterface::getOptions();
		_selected_option--;
		if (_selected_option<0)
			_selected_option=options.size()-1;
	} else if ((keypad==KEY_2)||(mouse_pressed==5)) {
		//Down
		vector<string> options = MenuInterface::getOptions();
		_selected_option++;
		if (_selected_option>=(int)options.size())
			_selected_option=0;
	}
}

/*
 * Returns the message to be displayed on the MenuInterfaceSurface
 */
vector<string> MenuInterface::getMessage() {
	vector<string> message;

	if (_page==0) {
		message.push_back("Random Lynx Games");
		message.push_back("presents");
	} else if (_page==1) {
		message.push_back("GALACTIC ARTIFACT");
		message.push_back("");
		message.push_back("Release C");
	} else if (_page==10) {
		message.push_back("INSTRUCTIONS");
		message.push_back("");
		message.push_back("Ingame instructions are not");
		message.push_back("available at this time.");
		message.push_back("");
		message.push_back("Please read the README");
		message.push_back("included with the archive.");
		message.push_back("or read the instructions");
		message.push_back("online at:");
		message.push_back("");
		message.push_back("http://games.randomlynx.net/");
	} else if (_page==20) {
		message.push_back("OPTIONS");
		message.push_back("");
		message.push_back("Game Options are not");
		message.push_back("available at this time.");
	} else if (_page==30) {
		message.push_back("ACHIEVEMENTS");
		message.push_back("");
		message.push_back("Eventually in game achivements");
		message.push_back("will go here.");
		message.push_back("");
		message.push_back("(You like to read all of");
		message.push_back("these, don't you?)");
	} else if (_page==40) {
		message.push_back("CREDITS");
		message.push_back("");
		message.push_back("...can be used to purchase");
		message.push_back("alien space candy from the");
		message.push_back("galactic vending machine");
		message.push_back("down the corridor...");

	}

	return message;
}

/*
 * Returns the options to be displayed on the MenuInferfaceSurface
 */
vector<string> MenuInterface::getOptions() {
	vector<string> options;

	switch (_page) {
		case 1:
			options.push_back("Start Network Game");
			options.push_back("Start Local Game");
			options.push_back("Start Local Game (Easy Mode)");
			options.push_back("Instructions");
			options.push_back("Options");
			options.push_back("Achievements");
			options.push_back("Credits");
			options.push_back("Exit");
			break;
		default:
			break;
	}

	return options;
}

/*
 * Returns the vertical position of the message to be rendered on the MenuInterfaceSurface
 */
int MenuInterface::getMessageVPos() {
	switch (_page) {
		case 1:
			return 160;
		default:
			break;
	}

	return -1;
}

/*
 * Returns the vertical position of the options to be displayed on the MenuInterfaceSurface
 */
int MenuInterface::getOptionsVPos() {
	switch (_page) {
		case 1:
			return 300;
		default:
			break;
	}

	return -1;
}

/*
 * Returns the option presently marked by the cursor
 */
int MenuInterface::getSelectedOption() {
	return _selected_option;
}

/*
 * Returns true if the message has changed to a new page.
 */
bool MenuInterface::isNewPage() {
	if (_new_page) {
		_new_page = false;
		return true;
	}
	return false;
}

/*
 * Returns true if the user has left the menu (either to start a game, or to quit)
 */
bool MenuInterface::isMenuExit() {
	return _menu_exit;
}

/*
 * Returns the game type:
 * -1: Quit Game
 * 0: Single Player
 * 1: Single Player (easy)
 * 2: Multiplayer
 */
int MenuInterface::getGameType() {
	return _game_type;
}

/*
 * Sets the game type.
 */
void MenuInterface::setGameType(int game_type) {
	_game_type=game_type;
}

/*
 * Resets Menu Interface
 */
void MenuInterface::reset() {
	//_page=1;
	_selected_option=0;
	_new_page=true;
	_menu_exit=false;
}

/*
 * Waits at the opening screen.
 */
void MenuInterface::longWait() {
	if (_page==0) {
		_page=1;
		_new_page=true;
	}

}
