/*
 * MessageBuffer
 *
 * A keystroke library that takes unicode keyboard events used in the game's chat window.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <locale.h>
#include <iostream>
#include <sstream>
#include "SDL/SDL.h"

#include "Messages.h"
#include "MessageBuffer.h"

using namespace std;

string MessageBuffer::_message;
bool MessageBuffer::_message_complete;
bool MessageBuffer::_quit;

/*
 * Constructor.  Creates new MessageBuffer for chat.
 */
MessageBuffer::MessageBuffer() {
	setlocale(LC_ALL, "");
	_message="";
	_message_complete=false;
	_quit=false;
}

/*
 * Copy Constructor.
 */
MessageBuffer::MessageBuffer(const MessageBuffer &mb) {
	_message=mb._message;
	_message_complete=mb._message_complete;
	_quit = mb._quit;
}

/*
 * Gets an individual keystroke to add to the buffer.
 * This method works in unicode, so there's no need to check for shift.
 * Even works on special German characters!
 */
void MessageBuffer::getKey() {
	SDL_Event event;

	char key_c='\0';

	if( SDL_PollEvent( &event ) ) {
		if ( event.type == SDL_KEYDOWN ) {
			if (event.key.keysym.unicode == SDLK_ESCAPE) {
				_message="";
				_message_complete=true;
			} else if (event.key.keysym.unicode != 0) {
				key_c = (char)event.key.keysym.unicode;
			}
		} else if (event.type==SDL_QUIT) {
			_quit=true;
		}
	}

	if (key_c!='\0') {
		if (key_c=='\r') { // Carriage Return
			_message_complete = true;
		} else if (key_c=='\b') { // Backspace
			if (_message.length()>0)
				_message = _message.substr(0,_message.length()-1);
		} else if (_message.length()<120) { // Have to max it out somewhere...
			_message += string(1, key_c);
		}
	}

}

/*
 * Returns true if the user has hit enter or escape to complete the message.
 */
bool MessageBuffer::isMessageComplete() {
	return _message_complete;
}

/*
 * Clears the message buffer (if the user hits escape).
 */
void MessageBuffer::clearMessage() {
	_message="";
	_message_complete=false;
}

/*
 * Gets the message buffer as a string.
 */
string MessageBuffer::getMessage() {
	return _message;
}

/*
 * Returns true if the user closes the window during chat.
 */
bool MessageBuffer::isQuit() {
	return _quit;
}
