/*
 * MessageSurface
 *
 * Renders game messages and user chat to the screen surface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <sstream>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "Ship.h"
#include "Messages.h"
#include "ScreenSurface.h"
#include "MessageBuffer.h"
#include "MessageSurface.h"

using namespace std;

int MessageSurface::_height_counter;

/*
 * Constructor.  Must be instantiated before blit().
 */
MessageSurface::MessageSurface() {
	_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_WIDTH-SCREEN_CENTER*2, SCREEN_CENTER*4/3-32-84, SCREEN_DEPTH,0,0,0,0);
	_text_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_WIDTH-SCREEN_CENTER*2, 20, SCREEN_DEPTH,0,0,0,0);


	_rect.x=SCREEN_CENTER*2;
	_rect.y=SCREEN_CENTER*2/3;
	_rect.w=SCREEN_WIDTH-SCREEN_CENTER*2;
	_rect.h=SCREEN_CENTER*4/3-84-32-20;

	_message_rect.x=0;
	_message_rect.y=0;
	_message_rect.w=SCREEN_WIDTH-SCREEN_CENTER*2;
	_message_rect.h=SCREEN_CENTER*4/3-84-32-20;

	_text_rect.x=SCREEN_CENTER*2;
	_text_rect.y=SCREEN_HEIGHT-84-32-20;
	_text_rect.w=SCREEN_WIDTH-SCREEN_CENTER*2;
	_text_rect.h=20;

	_height_counter=16;
}

/*
 * Destructor.  Frees surfaces.
 */
MessageSurface::~MessageSurface() {
	SDL_FreeSurface(_surface);
}

/*
 * Blits Messages and text chat to the screen surface.
 */
void MessageSurface::blit() {
	blitMessages();
	blitText();
}

/*
 * Blits scrolling messages to the Message Surface.
 */
void MessageSurface::blitMessages() {
	if (_height_counter<16) {
		_height_counter++;
	//FIXME NOT working
	//} else if (Messages::isUpdated()) {
	} else if (Messages::getMessageQueueSize()>0) {
		Messages::advanceMessageQueue();
		_height_counter=0;
	} else {
		return;
	}

	boxColor(_surface,0,0,SCREEN_CENTER*4/3,_rect.h,0x000000FF);

	Font::setFont("9x18B.fnt");

	for (int i=0;i<Messages::getMessagesSize();i++) {
		stringColor(_surface,5,14+_rect.h-17*(Messages::getMessagesSize())+17*i-_height_counter,Messages::getMessage(i).c_str(),Messages::getMessageColor(i));
	}

	Font::setFont("default");

	rectangleColor(_surface,0,0,SCREEN_CENTER*4/3-1,_rect.h-1,0x3F3FFFFF);

	SDL_BlitSurface(_surface,&_message_rect,ScreenSurface::getScreenSurface(),&_rect);
}

/*
 * Blits user chat to the message surface.
 */
void MessageSurface::blitText() {
	boxColor(_text_surface,0,0,SCREEN_WIDTH-SCREEN_CENTER*2, 20,0x000000FF);
	Font::setFont("9x18B.fnt");
	Ship (*my_ship) = Ship::getMyShip();
	string message = "> "+MessageBuffer::getMessage()+"_";
	int left_point=5;
	if (message.length()>34) {
		left_point-=9*(message.length()-34);
	}
	stringColor(_text_surface,left_point,0,message.c_str(),(*my_ship).getShipColor());
	Font::setFont("default");
	SDL_BlitSurface(_text_surface,NULL,ScreenSurface::getScreenSurface(),&_text_rect);
}
