/*
 * Messages
 *
 * Back end infrastructure for queueing and displaying game messages.
 * Game messages are queued and scroll upwards as they can be displayed.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef MESSAGES_H
#define MESSAGES_H

#include <string>
#include <vector>

#include "SDL/SDL.h"

class Messages {
private:
	static std::vector<std::string> _messages;
	static std::vector<std::string> _message_queue;
	static std::vector<Sint32> _messages_color;
	static std::vector<Sint32> _message_queue_color;
	static bool _updated;
public:
	Messages();
	static void addMessageToQueue(std::string s);
	static void addMessageToQueue(std::string s,Sint32 color);
	static int getMessagesSize();
	static int getMessageQueueSize();
	static void advanceMessageQueue();
	static std::string getMessage(const int i);
	static Sint32 getMessageColor(const int i);
	static bool isUpdated();
	static void reset();
};

#endif
