/*
 * MinimapSurface.
 *
 * Renders the minimap halo galaxy, level 0 stars and mini information surface to the screen surface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef MINIMAP_SURFACE_H
#define MINIMAP_SURFACE_H

#include <string>
#include <vector>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

class MinimapSurface {
private:
	static SDL_Surface *_surface;
	static SDL_Surface *_galaxy_surface;
	static SDL_Surface *_galaxy_halo;
	static SDL_Surface *_mini_information_surface;
	static SDL_Surface *_mini_border_surface;
	static SDL_Surface *_cursor_surface;
	static SDL_Surface *_artifact_cursor_surface;
	static SDL_Surface *_ship_surface;
	static SDL_Surface *_time_surface;
	static SDL_Surface *_fps_surface;
	static SDL_Rect _rect;
	static SDL_Rect _cursor_rect;
	static SDL_Rect _artifact_cursor_rect;
	static vector<SDL_Rect> _ship_rect;
	static SDL_Rect _time_rect;
	static SDL_Rect _fps_rect;
public:
	MinimapSurface();
	~MinimapSurface();
	static void blit();
	static void blitMiniGalaxy();
	static bool blitCursor();
	static bool blitArtifactCursor();
	static bool blitShip(Ship my_ship);
	static void blitShips();
	static void blitTime();
	static void blitFPS();
	static SDL_Surface *getMiniInformationSurface();
	static SDL_Surface *getMiniBorderSurface();


};

#endif
