/*
 * NetworkConnection
 *
 * Wrapper class for network communication, used by both client and server.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef NETWORK_CONNECTION_H
#define NETWORK_CONNECTION_H

#include <string>
#include <sstream>
#include <vector>
#include <list>
#include "SDL/SDL.h"
#include "SDL/SDL_net.h"

class NetworkConnection {
private:
	TCPsocket _socket;
	std::string _message_queue;
	std::string _ident;
	static NetworkConnection _client_network_connection;
	static bool _no_client_network_connection;
	static std::list<int> _used_ids;
	int _id;
	//int _last_sent;
	bool _not_temp;
	int _active;
	int _last_recv_time;
	int _last_send_time;
	int _last_ping_send_time;
public:
	NetworkConnection(TCPsocket socket);
	NetworkConnection(const NetworkConnection &nc);
	~NetworkConnection();
	bool sendMessage(std::string s);
	std::string receiveMessage();
	void queueMessage(std::string command);
	void queueMessage(std::string command,std::string message);
	std::string getMessageQueue();
	bool send();
	std::vector<std::string> receive();
	void destroy();
	void setIdent(std::string s);
	std::string getIdent();
	static NetworkConnection *getClientNetworkConnection();
	static void nullClientNetworkConnection();
	int getID();
	void closeConnection();
	static void initClient();
	std::string getIPAddress();
	bool isActive();
	void setActive(bool active);

	void setLastPingSendTime();
	void setLastRecvTime();
	int getTimeSinceLastPingSend();
	int getTimeSinceLastRecv();
	void sendPing();
};

#endif
