/*
 * NetworkEvent
 *
 * Client backend that handles server game messages.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include <string>

#include "SDL/SDL.h"

#include "Messages.h"
#include "Ship.h"
#include "MenuInterface.h"
#include "NetworkEvent.h"

using namespace std;

/*
 * Constructor
 */
NetworkEvent::NetworkEvent() {
}

/*
 * Handle message from game server.
 */
bool NetworkEvent::handleMessage(string whole_message) {
   	NetworkConnection *networkConnection = NetworkConnection::getClientNetworkConnection();
	string command=whole_message.substr(0,4);
	string message="";
	if (whole_message.length()>5)
		message = whole_message.substr(5,whole_message.length()-5);

	if (command=="MESG") {
		//Just a simple message.
		Messages::addMessageToQueue(message,0xBFBFBFFF);
	} else if (command=="BROA") {
		//Broadcast message to all users.
		Messages::addMessageToQueue(message,0xFFBFBFFF);
	} else if (command=="PONG") {
		//Receive PONG
		//Do nothing.
	} else if (command=="PING") {
		//Received PING, send PONG.
		(*networkConnection).queueMessage("PONG");
	} else if (command=="SHIP") {
		//Received SHIP event, some other user has moved.
		Ship::handleShipMessage(message);
	} else if (command=="JOIN") {
		//Received JOIN event, some other user has joined the game.
		(*Ship::getMyShip()).queueShipDataMessage();
		Ship::handleJoinMessage(message);
	} else if (command=="PLAY") {
		//Received PLAY Event.  Receiving data on other players names and numbers
		Ship::handlePlayMessage(message);
	} else if (command=="PART") {
		//Received PART Event.  Another player has left the game.
		Ship::handlePartMessage(message);
	} else if (command=="IDEN") {
		//Received IDEN event.  Data on *my* number and captain name.
		Ship::handleIdenMessage(message);
	} else if (command=="PLUN") {
		//Received PLUN event.  Another user has plundered a star.
		//These messages can get huge at the beginning of a network game, if you
		// join midway through.
		Star::handlePlunMessage(message);
	} else if (command=="VICT") {
		//Received VICT event.  Another player has won the game!
		Ship::handleVictMessage(message);
	} else if (command=="WAIT") {
		//Received WAIT event.  Server is sending out wait messages for network game restart.
		Messages::addMessageToQueue("Game will restart in "+message+" seconds.",0xFFBFBFFF);
	} else if (command=="ARTI") {
		//Received ARTI event.  Sets the location of the game artifact.
		//**NOTE!!** Yes, this is done very wrong.. artifacts should only be sent at the
		// end of the network game.  Hoping to fix this in a later release.
		Artifact::handleArtiMessage(message);
	} else if (command=="TALK") {
		//Received TALK message.  Another user is sending a chat message.
		Ship::handleTalkMessage(message);
	} else if (command=="KICK") {
		//Received KICK message.  We were just kicked from the server!!!
		(*networkConnection).queueMessage("PONG");
		NetworkConnection::nullClientNetworkConnection();
		Messages::addMessageToQueue("You have been kicked from the server (Reason: "+message+")");
		Messages::addMessageToQueue("Switching to single player local game.");
		Ship::removeAllShips();
		Ship *my_ship = Ship::getMyShip();
		(*my_ship).resetCaptainName();

		if (Artifact::isBetweenGames()) {
			Artifact::setBetweenGames(false);
		}

		MenuInterface::setGameType(0);

		Ship::initShipLocation();
		Artifact::initPrizeLocation();
	} else {
		Messages::addMessageToQueue("Unknown server message :'"+whole_message+"'",0x7F7F7FFF);
	}
	return true;
}
