/*
 * Ship
 *
 * Class describing a ship's present location and upgrades.  Handles events when stars are visited.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef SHIP_H
#define SHIP_H

#include <math.h>
#include <map>

#define UPGRADE_RADAR 0
#define UPGRADE_COMP  1
#define UPGRADE_SPEED 2
#define UPGRADE_TANK  3
#define UPGRADE_DRIVE  4
#define UPGRADE_CARGO 5

#include "Star.h"
#include "Coords.h"
#include "Map.h"

//class Star;
//class Coords;

class Ship {
private:
	int _upgrades[6];
	float _fuel;
	float _fuel_dist;
	float _cash;
	float _cash_dist;
	bool _updated;
	static Ship *_my_ship;
	Star _ship_star;
	int _ship_location[7];
	Coords _ship_coords;
	static bool _victory;
	static bool _failure;
	bool _arrived;
	bool _visiting_aliens;
	bool _recheck_planet;
	static map<const int, Ship, map_int_compare> _ships;
	int _ship_id;
	string _captain_name;
	static std::string _upgrade_name[6];
public:
	Ship();
	Ship(const Ship &s);
	float getCash();
	float getCashDist();
	float getFuel();
	float getFuelDist();
	int getRadar();
	float getFuelEfficiency();
	float getShipSpeed();
	int getCargoHoldSize();
	float getMaxFuel();
	void setCash(const float d);
	void addCash(const float d);
	void setCashDist(const float d);
	void setFuel(const float d);
	void addFuel(const float d);
	void setFuelDist(const float d);
	bool updateCashDist();
	bool updateFuelDist();
	int getUpgrades(const int i);
	void setUpgrades(const int i,const int j);
	void incrementUpgrades(const int i);
	void checkPlanet();
	bool isUpdated();
	void moveShip();
	static void moveShips();
	static void setMyShip(Ship &s);
	static Ship *getMyShip();
	static void initShipLocation();
	static void nullShipLocation();
	int *getShipLocation();
	void setShipLocation(int *ship_location);
	Coords getShipCoords();
	void setShipCoords(Coords ship_coords);
	Star getShipStar();
	void setShipStar(Star ship_star);
	void beckyCheat();
	static bool isVictory();
	static bool isFailure();
	static void resetVictoryFailure();
	bool isVisitingAliens();
	void setVisitingAliens(bool b);
	bool isMoving();
	void queueShipDataMessage();
	static void handleShipMessage(string message);
	static void handleTalkMessage(string message);
	static void handleVictMessage(string message);
	static void handleIdenMessage(string message);
	static void handlePartMessage(string message);
	static void handleJoinMessage(string message);
	static void handlePlayMessage(string message);
	static Ship *getShip(int id);
	void setShipID(int id);
	static void removeShip(int id);
	static void removeAllShips();
	static map<const int, Ship, map_int_compare> *getShips();
	Sint32 getShipColor();
	std::string getCaptainName();
	void reset();
	void resetCaptainName();
	static std::string *getUpgradeName();
};

#endif
