/*
 * ShipInterface
 *
 * Renders ship information (cash, fuel, upgrades) to the screen surface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <sstream>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "Ship.h"
#include "ScreenSurface.h"
#include "ShipSurface.h"

using namespace std;

/*
 * Constructor.  Prepares ship data surfaces.  Must be instantiatied before blit() is called.
 */
ShipSurface::ShipSurface() {
	_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_WIDTH-SCREEN_CENTER*2, 84, SCREEN_DEPTH,0,0,0,0);

	_rect.x=SCREEN_CENTER*2;
	_rect.y=SCREEN_HEIGHT-84;
	_rect.w=SCREEN_WIDTH-SCREEN_CENTER*2;
	_rect.h=100;

}

/*
 * Destructor.  Frees Ship data surface.
 */
ShipSurface::~ShipSurface() {
	SDL_FreeSurface(_surface);
}

/*
 * Blit's ship's data surface to screen surface.
 */
void ShipSurface::blit() {
	Ship *my_ship = Ship::getMyShip();
	if (!((*my_ship).isUpdated()))
		return;

	stringstream message;
	message.precision(2);

	int line_height=21;
	int char_width=9;

	string *upgrade_name = Ship::getUpgradeName();
	Sint32 upgrade_color[3] = {0xBF007FFF,0x007FBFFF,0x7FBF00FF};

	Font::setFont("9x18B.fnt");

	boxColor(_surface,0,0,SCREEN_WIDTH-2*SCREEN_CENTER,84,0x000000FF);

	boxColor(_surface,0,0,320,line_height-1,0x007F00FF);
	message.str(string());
	message << "Cash M$" << fixed << (*my_ship).getCashDist();
	stringColor(_surface,160-strlen(message.str().c_str())*char_width/2,1,message.str().c_str(),0xFFFFFFFF);

	boxColor(_surface,0,line_height,320,2*line_height-1,0x00003FFF);
	boxColor(_surface,0,line_height,(int)(320*((*my_ship).getFuelDist())/((*my_ship).getMaxFuel())),2*line_height-1,0x00007FFF);
	rectangleColor(_surface,0,line_height,320,2*line_height-1,0x00007FFF);
	message.str(string());
	//message.precision(0);
	message << "Fuel " << fixed << ((*my_ship).getFuelDist()) << "/" << (*my_ship).getMaxFuel() << " L";
	stringColor(_surface,160-strlen(message.str().c_str())*char_width/2,line_height+1,message.str().c_str(),0xFFFFFFFF);

	for (int i=0;i<3;i++) {
		for (int j=0;j<2;j++) {
			int k=j+2*i;
			boxColor(_surface,i*106,line_height*(j+2),(i+1)*106,line_height*(j+3)-1,0x00003FFF);
			boxColor(_surface,i*106,line_height*(j+2),i*106+(int)106*(*my_ship).getUpgrades(k)/10,line_height*(j+3)-1,upgrade_color[i]);
			rectangleColor(_surface,i*106,line_height*(j+2),(i+1)*106,line_height*(j+3)-1,0x00007FFF);
			message.str(string());
			message << upgrade_name[k] << " " << ((*my_ship).getUpgrades(k));
			stringColor(_surface,i*106+54-strlen(message.str().c_str())*char_width/2,1+line_height*(j+2),message.str().c_str(),0xFFFFFFFF);
		}
	}

	Font::setFont("default");

	rectangleColor(_surface,0,0,SCREEN_WIDTH-2*SCREEN_CENTER-1,83,0x3F3FFFFF);

	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
}
