/*
 * Star
 *
 * Object representing a single star in the galaxy.  Class also contains the static variable
 * _star, representing all stars in memory.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef STAR_H
#define STAR_H

#include "SDL/SDL.h"

#include "Coords.h"
#include "Artifact.h"

//class Coords;
//class Artifact;

#define SEED_STARS 5000
#define STAR_POWER 4

#define STARCLASS_FI   14
#define STARCLASS_GI   15
#define STARCLASS_KI   16
#define STARCLASS_MI   17
#define STARCLASS_FII  24
#define STARCLASS_GII  25
#define STARCLASS_KII  26
#define STARCLASS_MII  27
#define STARCLASS_FIII 34
#define STARCLASS_GIII 35
#define STARCLASS_KIII 36
#define STARCLASS_MIII 37
#define STARCLASS_FIV  44
#define STARCLASS_GIV  45
#define STARCLASS_KIV  46
#define STARCLASS_MIV  47
#define STARCLASS_OV   51
#define STARCLASS_BV   52
#define STARCLASS_AV   53
#define STARCLASS_FV   54
#define STARCLASS_GV   55
#define STARCLASS_KV   56
#define STARCLASS_MV   57
#define STARCLASS_UNKNOWN 99

using namespace std;

class Star {
private:
	int _index_number;
	int _star_number;
	int _level;
	Coords _position;
	int _star_class;
	bool _visible;
 	int _neighbour[10];
	bool _has_artifact;
	Artifact _artifact;

	static vector<Star> _star;
	static int _level_break[7];
	static vector<int> _plundered_stars;
	static vector<Star> _visible_stars;
public:
	Star();
	Star(const Star &s);
	Star(const int index_number, const int star_number, const int level, const int star_class, const Coords position, const bool has_artifact, const Artifact artifact);
	static Star nullStar();
	bool isNullStar();
	bool operator== (const Star &b);
	bool operator!= (const Star &b);
	bool isVisible();
	//FIXME - temp
	int getStarIndex();
	int getStarNumber();
	void setStarNumber(int star_number);
	string getStarClassName();
	int getStarClass();
	static int getStarClass(int star_number,int level);
	string getStarName();
	static void seedStars();
	static void surveyStars();
	static void isCloser(Star *i, Star *j, int direction);
	static Star sortStars(Coords offset);
	Star moveCursorStar(int direction);
	bool hasAliens();
	bool hasArtifact();
	static vector<Star> getVisibleAlienStars();
	static vector<Star> getVisibleArtifactStars();
	static void addStars (int location[], int level, Coords offset);
	static void addStars (int old_location[],int new_location[], int level);
	static void removeStars (int level);
	bool isPlundered();
	void plunder();
	static void queueStarPlunderMessage(int star_number);
	static void handlePlunMessage(std::string message);
	int getAlienType();
	static vector<Star> *getStars();
	static Star getStar(const int i);
	Coords getPosition();
	void setPosition(Coords pos);
	Sint32 getStarColor();
	Sint32 getStarColor(int level);
	Sint32 averageColor(Sint32 c1, Sint32 c2);
	int getSize();
	int getSize(int level);
	Artifact getArtifact();
	static void findVisible();
	static vector<Star> getVisibleStars();
	static Star getRandomStar(int location[],bool with_artifact);
	static Star getRandomStar2(int location[]);
	static void reset();
};

#endif
