/*
 * StringParser
 *
 * Helper class that takes network messages and extract their component variables.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <sstream>

#include "StringParser.h"

using namespace std;

/*
 * Takes a string, finds the first space, and pulls the string before that space from the string.
 * Used for parsing server messages.
 */
string StringParser::skimString(string &s) {
	string skim="";

	if (s.length()>0) {
		size_t found=s.find(" ");
		if (found!=string::npos) {
			skim = s.substr(0,found);
			string remainder = s.substr(found+1,s.length());
			s = remainder;
		} else {
			skim = s;
			s="";
		}
	}
	return skim;
}

/*
 * Takes a string, finds the first space, and pulls an integer before that space from the string.
 * Used for parsing server messages.
 */
int StringParser::skimInt(string &s) {
	string skim="";

	if (s.length()>0) {
		size_t found=s.find(" ");
		if (found!=string::npos) {
			skim = s.substr(0,found);
			string remainder = s.substr(found+1,s.length());
			s = remainder;
		} else {
			skim = s;
			s="";
		}
	}

	int i_skim;
	istringstream strin(skim);
	strin >> i_skim;

	if (strin)
		return i_skim;
	else
		return 0;
}

/*
 * Takes a string, finds the first space, and pulls the float before that space from the string.
 * Used for parsing server messages.
 */
float StringParser::skimFloat(string &s) {
	string skim="";

	if (s.length()>0) {
		size_t found=s.find(" ");
		if (found!=string::npos) {
			skim = s.substr(0,found);
			string remainder = s.substr(found+1,s.length());
			s = remainder;
		} else {
			skim = s;
			s="";
		}
	}

	float f_skim;
	istringstream strin(skim);
	strin >> f_skim;

	if (strin)
		return f_skim;
	else
		return 0.0f;
}
