/*
 * VerifyQuitSurface
 *
 * Displayed if the user hits ESC, asks the user if they really want to quit.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "ScreenSurface.h"
#include "VerifyQuitSurface.h"

using namespace std;

/*
 * Constructor.  Creates the Verify Quit Surface.  Must be instantiated before blit() is called.
 */
VerifyQuitSurface::VerifyQuitSurface() {
	_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, 300, 100, SCREEN_DEPTH,0,0,0,0);
	_translucency_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, 300, 100, SCREEN_DEPTH,0,0,0,0);

	SDL_SetColorKey(_surface, SDL_SRCCOLORKEY, 0x00000000);
	SDL_SetAlpha(_translucency_surface,SDL_SRCALPHA,127);

	_rect.x=SCREEN_CENTER-150;
	_rect.y=SCREEN_CENTER-50;
	_rect.w=300;
	_rect.h=100;
}

/*
 * Destructor. Frees surfaces.
 */
VerifyQuitSurface::~VerifyQuitSurface() {
	SDL_FreeSurface(_surface);
	SDL_FreeSurface(_translucency_surface);
}

/*
 * Blits Verify Quit Surface to screen.
 */
void VerifyQuitSurface::blit() {
	boxColor(_surface,0,0,SCREEN_CENTER*2,SCREEN_CENTER*2,0x000000FF);
	boxColor(_translucency_surface,0,0,SCREEN_CENTER*2,SCREEN_CENTER*2,0x3F3F3FFF);
	blitMessage();
	SDL_BlitSurface(_translucency_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
}

/*
 * Blits Verify Quit Message to Verify Quit surface.
 */
void VerifyQuitSurface::blitMessage() {
	Font::setFont("9x18B.fnt");

	string message = "Do you want to quit? (Y/N)";

	rectangleColor(_surface,0,0,_rect.w-1,_rect.h-1,0xFFFFFFFF);
	stringColor(_surface,_rect.w/2-9*message.length()/2,_rect.h/2-18/2,message.c_str(),0xFFFFFFFF);

	Font::setFont("default");
}
