/*
 * View
 *
 * Class representing what the user is looking at, and where the user will be looking at soon.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#ifndef VIEW_H
#define VIEW_H

class Coords;
class KeyPressed;
class Star;

class View {
private:
	static Coords _offset;
	static Coords _offset_dest;
	static float _scale;
	static float _scale_dest;
	static int _location[7];
	static int _level;
	static bool _updated;
	static bool _update_override;

	static Star _cursor_star;
	static Coords _cursor_coords;
	static bool _cursor_star_set;
public:
	View();
	View (View &v);
	static void reset();
	static Coords getOffset();
	static Coords getOffsetDest();
	static void setOffset(const Coords c);
	static float getScale();
	static float getScaleDest();
	static void setScale(const float d);
	static int *getLocation();
	static void setLocation(const int location[7]);
	static int getLevel();
	static void setLevel(int i);
	static bool isUpdated();
	static void setUpdateOverride();
	static bool isUpdateOverride();
	static void resetOverride();
	static void incrementTime();
	static Coords getOffset(int location[], int level);
	static Coords getOffset(int location[]);
	static int getLevel(int location[]);
	static int getLevel(int location);
	static float getScale(int level);
	static Coords getOffset(int location);
	static bool moveLocation(int location[], const int level, const int direction);
	static void keyPressed(KeyPressed kp);
	static void mousePressed(KeyPressed kp);

	static Coords getCursorCoords();
	static void setCursorCoords(Coords cursor_coords);
	static Star getCursorStar();
	static void setCursorStar(Star cursor_star);
	static bool isZoomedIn();
	static bool isAlmostZoomedIn();
	static bool isMoving();
	static bool isZooming();
};

#endif
